@@ -2,20 +2,13 @@
    Copyright (c) 2011, 2016, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License, version 2.0,
-   as published by the Free Software Foundation.
-
-   This program is also distributed with certain software (including
-   but not limited to OpenSSL) that is licensed under separate terms,
-   as designated in a particular file or component or in included license
-   documentation.  The authors of MySQL hereby grant you an additional
-   permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License, version 2.0, for more details.
+   GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
@@ -30,6 +23,10 @@
 #include <stdio.h>
 #include <string.h>
 
+#ifdef _WIN32
+#define popen _popen
+#define pclose _pclose
+#endif
 
 #define SHOW_VERSION "1.0.0"
 #define PRINT_VERSION do { printf("%s  Ver %s Distrib %s\n",    \
@@ -106,6 +103,9 @@ int main(int argc,char *argv[])
   char server_path[FN_REFLEN];
   char operation[16];
 
+  fprintf(stderr, "Warning: mysql_plugin is deprecated and will be removed in a future "
+          "version.\n");
+
   MY_INIT(argv[0]);
   plugin_data.name= 0; // initialize name
   
@@ -194,13 +194,14 @@ static char *get_value(char *line, const char *item)
   int item_len= (int)strlen(item);
   int line_len = (int)strlen(line);
 
-  if ((strncasecmp(line, item, item_len) == 0))
+  if ((native_strncasecmp(line, item, item_len) == 0))
   {
     int start= 0;
     char *s= 0;
 
     s = line + item_len + 1;
-    destination= my_strndup(s, line_len - start, MYF(MY_FAE));
+    destination= my_strndup(PSI_NOT_INSTRUMENTED,
+                            s, line_len - start, MYF(MY_FAE));
     destination[line_len - item_len - 2]= 0;
   }
   return destination;
@@ -242,7 +243,7 @@ static int run_command(char* cmd, const char *mode)
 }
 
 
-#ifdef __WIN__
+#ifdef _WIN32
 /**
   Check to see if there are spaces in a path.
   
@@ -268,9 +269,10 @@ static int has_spaces(const char *path)
 static char *convert_path(const char *argument)
 {
   /* Convert / to \\ to make Windows paths */
-  char *winfilename= my_strdup(argument, MYF(MY_FAE));
+  char *winfilename= my_strdup(PSI_NOT_INSTRUMENTED,
+                               argument, MYF(MY_FAE));
   char *pos, *end;
-  int length= strlen(argument);
+  size_t length= strlen(argument);
 
   for (pos= winfilename, end= pos+length ; pos < end ; pos++)
   {
@@ -295,12 +297,13 @@ static char *add_quotes(const char *path)
   char windows_cmd_friendly[FN_REFLEN];
 
   if (has_spaces(path))
-    snprintf(windows_cmd_friendly, sizeof(windows_cmd_friendly),
-             "\"%s\"", path);
+    my_snprintf(windows_cmd_friendly, sizeof(windows_cmd_friendly),
+                "\"%s\"", path);
   else
-    snprintf(windows_cmd_friendly, sizeof(windows_cmd_friendly),
-             "%s", path);
-  return my_strdup(windows_cmd_friendly, MYF(MY_FAE));
+    my_snprintf(windows_cmd_friendly, sizeof(windows_cmd_friendly),
+                "%s", path);
+  return my_strdup(PSI_NOT_INSTRUMENTED,
+                   windows_cmd_friendly, MYF(MY_FAE));
 }
 #endif
 
@@ -340,7 +343,7 @@ static int get_default_values()
     if ((error= make_tempfile(defaults_file, "txt")))
       goto exit;
 
-#ifdef __WIN__
+#ifdef _WIN32
     {
       char *format_str= 0;
   
@@ -349,16 +352,16 @@ static int get_default_values()
       else
         format_str = "%s mysqld > %s";
   
-      snprintf(defaults_cmd, sizeof(defaults_cmd), format_str,
-               add_quotes(tool_path), add_quotes(defaults_file));
+      my_snprintf(defaults_cmd, sizeof(defaults_cmd), format_str,
+                  add_quotes(tool_path), add_quotes(defaults_file));
       if (opt_verbose)
       {
         printf("# my_print_defaults found: %s\n", tool_path);
       }
     }
 #else
-    snprintf(defaults_cmd, sizeof(defaults_cmd),
-             format_cmd, tool_path, defaults_file);
+    my_snprintf(defaults_cmd, sizeof(defaults_cmd),
+                format_cmd, tool_path, defaults_file);
 #endif
 
     /* Execute the command */
@@ -381,19 +384,23 @@ static int get_default_values()
 
       if ((opt_datadir == 0) && ((value= get_value(line, "--datadir"))))
       {
-        opt_datadir= my_strdup(value, MYF(MY_FAE));
+        opt_datadir= my_strdup(PSI_NOT_INSTRUMENTED,
+                               value, MYF(MY_FAE));
       }
       if ((opt_basedir == 0) && ((value= get_value(line, "--basedir"))))
       {
-        opt_basedir= my_strdup(value, MYF(MY_FAE));
+        opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED,
+                               value, MYF(MY_FAE));
       }
       if ((opt_plugin_dir == 0) && ((value= get_value(line, "--plugin_dir"))))
       {
-        opt_plugin_dir= my_strdup(value, MYF(MY_FAE));
+        opt_plugin_dir= my_strdup(PSI_NOT_INSTRUMENTED,
+                                  value, MYF(MY_FAE));
       }
       if ((opt_plugin_ini == 0) && ((value= get_value(line, "--plugin_ini"))))
       {
-        opt_plugin_ini= my_strdup(value, MYF(MY_FAE));
+        opt_plugin_ini= my_strdup(PSI_NOT_INSTRUMENTED,
+                                  value, MYF(MY_FAE));
       }
     }
   }
@@ -503,22 +510,28 @@ get_one_option(int optid,
     usage();
     exit(0);
   case 'd':
-    opt_datadir= my_strdup(argument, MYF(MY_FAE));
+    opt_datadir= my_strdup(PSI_NOT_INSTRUMENTED,
+                           argument, MYF(MY_FAE));
     break;
   case 'b':
-    opt_basedir= my_strdup(argument, MYF(MY_FAE));
+    opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED,
+                           argument, MYF(MY_FAE));
     break;
   case 'p':
-    opt_plugin_dir= my_strdup(argument, MYF(MY_FAE));
+    opt_plugin_dir= my_strdup(PSI_NOT_INSTRUMENTED,
+                              argument, MYF(MY_FAE));
     break;
   case 'i':
-    opt_plugin_ini= my_strdup(argument, MYF(MY_FAE));
+    opt_plugin_ini= my_strdup(PSI_NOT_INSTRUMENTED,
+                              argument, MYF(MY_FAE));
     break;
   case 'm':
-    opt_mysqld= my_strdup(argument, MYF(MY_FAE));
+    opt_mysqld= my_strdup(PSI_NOT_INSTRUMENTED,
+                          argument, MYF(MY_FAE));
     break;
   case 'f':
-    opt_my_print_defaults= my_strdup(argument, MYF(MY_FAE));
+    opt_my_print_defaults= my_strdup(PSI_NOT_INSTRUMENTED,
+                                     argument, MYF(MY_FAE));
     break;
   }
   return 0;
@@ -561,7 +574,7 @@ static int search_dir(const char * base_path, const char *tool_name,
 {
   char new_path[FN_REFLEN];
   char source_path[FN_REFLEN];
-#if __WIN__
+#if _WIN32
   char win_abs_path[FN_REFLEN];
   char self_name[FN_REFLEN];
   const char *last_fn_libchar;
@@ -581,7 +594,7 @@ static int search_dir(const char * base_path, const char *tool_name,
     return 0;
   }  
 
-#if __WIN__
+#if _WIN32
   /*
     On Windows above code will not be able to find the file since
     path names are not absolute and file_exists works only with 
@@ -662,7 +675,8 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   if (opt_plugin_ini == 0)
   {
     fn_format(path, config_file, opt_plugin_dir, "", MYF(0));
-    opt_plugin_ini= my_strdup(path, MYF(MY_FAE));
+    opt_plugin_ini= my_strdup(PSI_NOT_INSTRUMENTED,
+                              path, MYF(MY_FAE));
   }
   if (!file_exists(opt_plugin_ini))
   {
@@ -678,7 +692,8 @@ static int load_plugin_data(char *plugin_name, char *config_file)
   }
 
   /* save name */
-  plugin_data.name= my_strdup(plugin_name, MYF(MY_WME));
+  plugin_data.name= my_strdup(PSI_NOT_INSTRUMENTED,
+                              plugin_name, MYF(MY_WME));
 
   /* Read plugin components */
   while (i < 16)
@@ -708,14 +723,16 @@ static int load_plugin_data(char *plugin_name, char *config_file)
       /* Add proper file extension for soname */
       strcat(line, FN_SOEXT);
       /* save so_name */
-      plugin_data.so_name= my_strdup(line, MYF(MY_WME|MY_ZEROFILL));
+      plugin_data.so_name= my_strdup(PSI_NOT_INSTRUMENTED,
+                                     line, MYF(MY_WME|MY_ZEROFILL));
       i++;
     }
     else
     {
       if (strlen(line) > 0)
       {
-        plugin_data.components[i]= my_strdup(line, MYF(MY_WME));
+        plugin_data.components[i]= my_strdup(PSI_NOT_INSTRUMENTED,
+                                             line, MYF(MY_WME));
         i++;
       }
       else
@@ -758,11 +775,11 @@ static int check_options(int argc, char **argv, char *operation)
   
   /* Form prefix strings for the options. */
   const char *basedir_prefix = "--basedir=";
-  int basedir_len= strlen(basedir_prefix);
+  size_t basedir_len= strlen(basedir_prefix);
   const char *datadir_prefix = "--datadir=";
-  int datadir_len= strlen(datadir_prefix);
+  size_t datadir_len= strlen(datadir_prefix);
   const char *plugin_dir_prefix = "--plugin_dir=";
-  int plugin_dir_len= strlen(plugin_dir_prefix);
+  size_t plugin_dir_len= strlen(plugin_dir_prefix);
 
   strcpy(plugin_name, "");
   for (i = 0; i < argc && num_found < 5; i++)
@@ -772,30 +789,33 @@ static int check_options(int argc, char **argv, char *operation)
     {
       continue;
     }
-    if ((strcasecmp(argv[i], "ENABLE") == 0) ||
-        (strcasecmp(argv[i], "DISABLE") == 0))
+    if ((native_strcasecmp(argv[i], "ENABLE") == 0) ||
+        (native_strcasecmp(argv[i], "DISABLE") == 0))
     {
       strcpy(operation, argv[i]);
       num_found++;
     }
-    else if ((strncasecmp(argv[i], basedir_prefix, basedir_len) == 0) &&
+    else if ((native_strncasecmp(argv[i], basedir_prefix, basedir_len) == 0) &&
              !opt_basedir)
     {
-      opt_basedir= my_strndup(argv[i]+basedir_len,
+      opt_basedir= my_strndup(PSI_NOT_INSTRUMENTED,
+                              argv[i]+basedir_len,
                               strlen(argv[i])-basedir_len, MYF(MY_FAE));
       num_found++;
     }
-    else if ((strncasecmp(argv[i], datadir_prefix, datadir_len) == 0) &&
+    else if ((native_strncasecmp(argv[i], datadir_prefix, datadir_len) == 0) &&
              !opt_datadir)
     {
-      opt_datadir= my_strndup(argv[i]+datadir_len,
+      opt_datadir= my_strndup(PSI_NOT_INSTRUMENTED,
+                              argv[i]+datadir_len,
                               strlen(argv[i])-datadir_len, MYF(MY_FAE));
       num_found++;
     }
-    else if ((strncasecmp(argv[i], plugin_dir_prefix, plugin_dir_len) == 0) &&
+    else if ((native_strncasecmp(argv[i], plugin_dir_prefix, plugin_dir_len) == 0) &&
              !opt_plugin_dir)
     {
-      opt_plugin_dir= my_strndup(argv[i]+plugin_dir_len,
+      opt_plugin_dir= my_strndup(PSI_NOT_INSTRUMENTED,
+                                 argv[i]+plugin_dir_len,
                                  strlen(argv[i])-plugin_dir_len, MYF(MY_FAE));
       num_found++;
     }
@@ -837,7 +857,7 @@ static int check_options(int argc, char **argv, char *operation)
     {
       return 1;
     }
-    if (strcasecmp(plugin_data.name, plugin_name) != 0)
+    if (native_strcasecmp(plugin_data.name, plugin_name) != 0)
     {
       fprintf(stderr, "ERROR: plugin name requested does not match config "
               "file data.\n");
@@ -895,20 +915,21 @@ static int process_options(int argc, char *argv[], char *operation)
   if (opt_basedir)
   {
     i= (int)strlength(opt_basedir);
-    if (opt_basedir[i-1] != FN_LIBCHAR || opt_basedir[i-1] != FN_LIBCHAR2)
+    if (!is_directory_separator(opt_basedir[i-1]))
     {
       char buff[FN_REFLEN];
       memset(buff, 0, sizeof(buff));
       
       strncpy(buff, opt_basedir, sizeof(buff) - 1);
-#ifdef __WIN__
+#ifdef _WIN32
       strncat(buff, "/", sizeof(buff) - strlen(buff) - 1);
 #else
       strncat(buff, FN_DIRSEP, sizeof(buff) - strlen(buff) - 1);
 #endif
       buff[sizeof(buff) - 1]= 0;
       my_free(opt_basedir);
-      opt_basedir= my_strdup(buff, MYF(MY_FAE));
+      opt_basedir= my_strdup(PSI_NOT_INSTRUMENTED,
+                             buff, MYF(MY_FAE));
     }
   }
   
@@ -1068,7 +1089,7 @@ static int find_plugin(char *tp_path)
 
 
 /**
-  Build the boostrap file.
+  Build the bootstrap file.
   
   Create a new file and populate it with SQL commands to ENABLE or DISABLE
   the plugin via REPLACE and DELETE operations on the mysql.plugin table.
@@ -1111,7 +1132,7 @@ static int build_bootstrap_file(char *operation, char *bootstrap)
     error= 1;
     goto exit;
   }
-  if (strcasecmp(operation, "enable") == 0)
+  if (native_strcasecmp(operation, "enable") == 0)
   {
     int i= 0;
     fprintf(file, "REPLACE INTO mysql.plugin VALUES ");
@@ -1197,7 +1218,7 @@ exit:
   
   Create a command line sequence to launch mysqld in bootstrap mode. This
   will allow mysqld to launch a minimal server instance to read and
-  execute SQL commands from a file piped in (the boostrap file). We use
+  execute SQL commands from a file piped in (the bootstrap file). We use
   the --no-defaults option to skip reading values from the config file.
 
   The bootstrap mode skips loading of plugins and many other subsystems.
@@ -1220,7 +1241,7 @@ static int bootstrap_server(char *server_path, char *bootstrap_file)
   char bootstrap_cmd[FN_REFLEN * 2 + sizeof(format_cmd)];
   int error= 0;
 
-#ifdef __WIN__
+#ifdef _WIN32
   char *format_str= 0;
   const char *verbose_str= NULL;
    
@@ -1235,13 +1256,13 @@ static int bootstrap_server(char *server_path, char *bootstrap_file)
   else 
     format_str= "%s %s --bootstrap --datadir=%s --basedir=%s < %s";
 
-  snprintf(bootstrap_cmd, sizeof(bootstrap_cmd), format_str,
-           add_quotes(convert_path(server_path)), verbose_str,
-           add_quotes(opt_datadir), add_quotes(opt_basedir),
-           add_quotes(bootstrap_file));
+  my_snprintf(bootstrap_cmd, sizeof(bootstrap_cmd), format_str,
+              add_quotes(convert_path(server_path)), verbose_str,
+              add_quotes(opt_datadir), add_quotes(opt_basedir),
+              add_quotes(bootstrap_file));
 #else
-  snprintf(bootstrap_cmd, sizeof(bootstrap_cmd),
-           format_cmd, server_path, opt_datadir, opt_basedir, bootstrap_file);
+  my_snprintf(bootstrap_cmd, sizeof(bootstrap_cmd), format_cmd,
+              server_path, opt_datadir, opt_basedir, bootstrap_file);
 #endif
 
   /* Execute the command */
