@@ -9,6 +9,7 @@
 #include "gu_throw.hpp"
 
 #include <map>
+#include <algorithm> // std::for_each
 
 using namespace galera;
 
@@ -587,16 +588,6 @@ galera::Certification::do_test_v3(TrxHandle* trx, bool store_keys)
     long const      key_count(key_set.count());
     long            processed(0);
 
-    // This check if the keys are appended.
-    // Generally almost all cases keys are appeneded before certification
-    // except in case of CTAS which is executed non-TOI fashion due to
-    // involvement of SELECT. If the source table is empty then no key
-    // is appended and so key_count == 0.
-    // Unfortunately there is no way to check if the original SQL statement
-    // is of type CTAS type but leaving this assert can cause random failure
-    // so we will disable it for now.
-    // assert(key_count > 0);
-
     key_set.rewind();
 
     for (; processed < key_count; ++processed)
@@ -817,7 +808,7 @@ galera::Certification::do_test_preordered(TrxHandle* trx)
 }
 
 
-galera::Certification::Certification(gu::Config& conf, ServiceThd& thd)
+galera::Certification::Certification(gu::Config& conf, ServiceThd& thd, gcache::GCache& gcache)
     :
     version_               (-1),
     trx_map_               (),
@@ -825,6 +816,7 @@ galera::Certification::Certification(gu::Config& conf, ServiceThd& thd)
     cert_index_ng_         (),
     deps_set_              (),
     service_thd_           (thd),
+    gcache_                (gcache),
 #ifdef HAVE_PSI_INTERFACE
     mutex_                 (WSREP_PFS_INSTR_TAG_CERT_MUTEX),
 #else
@@ -858,17 +850,17 @@ galera::Certification::Certification(gu::Config& conf, ServiceThd& thd)
 
 galera::Certification::~Certification()
 {
-    log_info << "cert index usage at exit "   << cert_index_.size();
-    log_info << "cert trx map usage at exit " << trx_map_.size();
-    log_info << "deps set usage at exit "     << deps_set_.size();
+    log_debug << "cert index usage at exit "   << cert_index_.size();
+    log_debug << "cert trx map usage at exit " << trx_map_.size();
+    log_debug << "deps set usage at exit "     << deps_set_.size();
 
     double avg_cert_interval(0);
     double avg_deps_dist(0);
     size_t index_size(0);
     stats_get(avg_cert_interval, avg_deps_dist, index_size);
-    log_info << "avg deps dist "              << avg_deps_dist;
-    log_info << "avg cert interval "          << avg_cert_interval;
-    log_info << "cert index size "            << index_size;
+    log_debug << "avg deps dist "              << avg_deps_dist;
+    log_debug << "avg cert interval "          << avg_cert_interval;
+    log_debug << "cert index size "            << index_size;
 
     gu::Lock lock(mutex_);
 
@@ -918,8 +910,6 @@ void galera::Certification::assign_initial_position(wsrep_seqno_t seqno,
     }
 
     trx_map_.clear();
-    service_thd_.release_seqno(position_);
-    service_thd_.flush();
 
     log_info << "Assign initial position for certification: " << seqno
              << ", protocol version: " << version;
@@ -1085,7 +1075,7 @@ wsrep_seqno_t galera::Certification::set_trx_committed(TrxHandle* trx)
             deps_set_.erase(i);
         }
 
-        if (gu_unlikely(index_purge_required()))
+        if (gu_unlikely(gcache_.cleanup_required() || index_purge_required()))
         {
             ret = get_safe_to_discard_seqno_();
         }
