@@ -139,6 +139,7 @@ galera::ReplicatorSMM::ReplicatorSMM(const struct wsrep_init_args* args)
     safe_to_bootstrap_  (true),
     trx_params_         (config_.get(BASE_DIR), -1,
                          KeySet::version(config_.get(Param::key_format)),
+                         TrxHandle::Defaults.record_set_ver_,
                          gu::from_string<int>(config_.get(
                              Param::max_write_set_size))),
     uuid_               (WSREP_UUID_UNDEFINED),
@@ -304,6 +305,7 @@ galera::ReplicatorSMM::~ReplicatorSMM()
     case S_SYNCED:
     case S_DONOR:
         close();
+        // fall through
     case S_CLOSING:
         // @todo wait that all users have left the building
     case S_CLOSED:
@@ -518,6 +520,9 @@ void galera::ReplicatorSMM::apply_trx(void* recv_ctx, TrxHandle* trx)
     if (gu_likely(co_mode_ != CommitOrder::BYPASS) && trx->is_toi())
     {
         gu_trace(commit_monitor_.leave(co));
+
+        // Allow tests to block the applier thread using the DBUG facilities
+        GU_DBUG_SYNC_WAIT("sync.apply_trx.after_commit_leave");
     }
     trx->set_state(TrxHandle::S_COMMITTED);
 
@@ -792,21 +797,31 @@ wsrep_status_t galera::ReplicatorSMM::pre_commit(TrxHandle*        trx,
     This means transaction can get aborted on completion of replicate
     before pre-commit action start. This condition capture that scenario
     and ensure that resources are released. */
-    if (trx->state() == TrxHandle::S_MUST_ABORT ||
-        trx->state() == TrxHandle::S_ABORTING)
+    if (trx->state() == TrxHandle::S_MUST_ABORT)
     {
-        LocalOrder lo(*trx);
-        ApplyOrder ao(*trx);
-        CommitOrder co(*trx, co_mode_);
+        wsrep_status_t retval(WSREP_PRECOMMIT_ABORT);
+        retval = cert_for_aborted(trx);
 
-        local_monitor_.self_cancel(lo);
+        if (retval != WSREP_BF_ABORT)
+        {
+            LocalOrder  lo(*trx);
+            ApplyOrder  ao(*trx);
+            CommitOrder co(*trx, co_mode_);
+            local_monitor_.self_cancel(lo);
+            apply_monitor_.self_cancel(ao);
+            if (co_mode_ !=CommitOrder::BYPASS) commit_monitor_.self_cancel(co);
+        }
+        else if (meta != 0)
+        {
+            meta->gtid.uuid  = state_uuid_;
+            meta->gtid.seqno = trx->global_seqno();
+            meta->depends_on = trx->depends_seqno();
+        }
 
-        gcache_.seqno_assign (trx->action(), trx->global_seqno(), -1);
+        if (trx->state() == TrxHandle::S_MUST_ABORT)
+            trx->set_state(TrxHandle::S_ABORTING);
 
-        apply_monitor_.self_cancel(ao);
-        if (co_mode_ != CommitOrder::BYPASS) commit_monitor_.self_cancel(co);
-
-        return WSREP_PRECOMMIT_ABORT;
+        return retval;
     }
 
     assert(trx->state() == TrxHandle::S_REPLICATING);
@@ -932,8 +947,8 @@ wsrep_status_t galera::ReplicatorSMM::replay_trx(TrxHandle* trx, void* trx_ctx)
         ApplyOrder ao(*trx);
         gu_trace(apply_monitor_.enter(ao));
         trx->set_state(TrxHandle::S_MUST_REPLAY_CM);
-        // fall through
     }
+    // fall through
     case TrxHandle::S_MUST_REPLAY_CM:
         if (co_mode_ != CommitOrder::BYPASS)
         {
@@ -980,6 +995,13 @@ wsrep_status_t galera::ReplicatorSMM::replay_trx(TrxHandle* trx, void* trx_ctx)
             log_fatal << "Failed to re-apply trx: " << *trx;
             log_fatal << e.what();
             log_fatal << "Node consistency compromized, aborting...";
+
+            /* Before doing a graceful exit ensure that node isolate itself
+            from the cluster. This will cause the quorum to re-evaluate
+            and if minority nodes are left with different set of data
+            they can turn non-Primary to avoid further data consistency issue. */
+            param_set("gmcast.isolate", "1");
+
             abort();
         }
 
@@ -1015,7 +1037,13 @@ wsrep_status_t galera::ReplicatorSMM::interim_commit(TrxHandle* trx)
     assert(trx->local_seqno() > -1 && trx->global_seqno() > -1);
 
     CommitOrder co(*trx, co_mode_);
-    if (co_mode_ != CommitOrder::BYPASS) commit_monitor_.leave(co);
+    if (co_mode_ != CommitOrder::BYPASS)
+    {
+        commit_monitor_.leave(co);
+
+        // Allow tests to block the applier thread using the DBUG facilities
+        GU_DBUG_SYNC_WAIT("sync.interim_commit.after_commit_leave");
+    }
     trx->mark_interim_committed(true);
 
     return WSREP_OK;
@@ -1044,6 +1072,9 @@ wsrep_status_t galera::ReplicatorSMM::post_commit(TrxHandle* trx)
     {
         CommitOrder co(*trx, co_mode_);
         if (co_mode_ != CommitOrder::BYPASS) commit_monitor_.leave(co);
+
+        // Allow tests to block the applier thread using the DBUG facilities
+        GU_DBUG_SYNC_WAIT("sync.post_commit.after_commit_leave");
     }
     trx->mark_interim_committed(false);
 
@@ -1102,14 +1133,9 @@ wsrep_status_t galera::ReplicatorSMM::causal_read(wsrep_gtid_t* gtid)
         // finished.
         gu::datetime::Date wait_until(gu::datetime::Date::calendar()
                                       + causal_read_timeout_);
-        if (gu_likely(co_mode_ != CommitOrder::BYPASS))
-        {
-            commit_monitor_.wait(cseq, wait_until);
-        }
-        else
-        {
-            apply_monitor_.wait(cseq, wait_until);
-        }
+
+        apply_monitor_.wait(cseq, wait_until);
+
         if (gtid != 0)
         {
             gtid->uuid = state_uuid_;
@@ -1120,7 +1146,7 @@ wsrep_status_t galera::ReplicatorSMM::causal_read(wsrep_gtid_t* gtid)
     }
     catch (gu::Exception& e)
     {
-        log_debug << "monitor wait failed for causal read: " << e.what();
+        log_info << "monitor wait failed for causal read (repl.causal_read_timeout): " << e.what();
         return WSREP_TRX_FAIL;
     }
 }
@@ -1190,7 +1216,11 @@ wsrep_status_t galera::ReplicatorSMM::to_isolation_end(TrxHandle* trx)
     log_debug << "Done executing TO isolated action: " << *trx;
 
     CommitOrder co(*trx, co_mode_);
-    if (co_mode_ != CommitOrder::BYPASS) commit_monitor_.leave(co);
+    if (co_mode_ != CommitOrder::BYPASS)
+    {
+        commit_monitor_.leave(co);
+        GU_DBUG_SYNC_WAIT("sync.to_isolation_end.after_commit_leave");
+    }
     ApplyOrder ao(*trx);
     report_last_committed(cert_.set_trx_committed(trx));
     apply_monitor_.leave(ao);
@@ -1217,8 +1247,9 @@ writeset_from_handle (wsrep_po_handle_t& handle,
 //                gu::String<256>(trx_params.working_dir_) << '/' << &handle,
                 trx_params.working_dir_, wsrep_trx_id_t(&handle),
                 /* key format is not essential since we're not adding keys */
-                KeySet::version(trx_params.key_format_), NULL, 0,
-                0, WriteSetNG::MAX_VERSION, DataSet::MAX_VERSION, DataSet::MAX_VERSION,
+                KeySet::version(trx_params.key_format_), NULL, 0, 0,
+                trx_params.record_set_ver_,
+                WriteSetNG::MAX_VERSION, DataSet::MAX_VERSION, DataSet::MAX_VERSION,
                 trx_params.max_write_set_size_);
 
             handle.opaque = ret;
@@ -1379,7 +1410,14 @@ void galera::ReplicatorSMM::process_trx(void* recv_ctx, TrxHandle* trx)
 
             log_fatal << "Failed to apply trx: " << *trx;
             log_fatal << e.what();
-            log_fatal << "Node consistency compromized, aborting...";
+            log_fatal << "Node consistency compromised, aborting...";
+
+            /* Before doing a graceful exit ensure that node isolate itself
+            from the cluster. This will cause the quorum to re-evaluate
+            and if minority nodes are left with different set of data
+            they can turn non-Primary to avoid further data consistency issue. */
+            param_set("gmcast.isolate", "1");
+
             abort();
         }
         break;
@@ -1416,6 +1454,8 @@ void galera::ReplicatorSMM::process_commit_cut(wsrep_seqno_t seq,
 
 void galera::ReplicatorSMM::establish_protocol_versions (int proto_ver)
 {
+    trx_params_.record_set_ver_ = gu::RecordSet::VER1;
+
     switch (proto_ver)
     {
     case 1:
@@ -1446,6 +1486,12 @@ void galera::ReplicatorSMM::establish_protocol_versions (int proto_ver)
         trx_params_.version_ = 3;
         str_proto_ver_ = 2;
         break;
+    case 8:
+        // Protocol upgrade to enforce 8-byte alignment in writesets.
+        trx_params_.version_ = 3;
+        trx_params_.record_set_ver_ = gu::RecordSet::VER2;
+        str_proto_ver_ = 2;
+        break;
     default:
         log_fatal << "Configuration change resulted in an unsupported protocol "
             "version: " << proto_ver << ". Can't continue.";
@@ -1554,6 +1600,12 @@ galera::ReplicatorSMM::process_conf_change(void*                    recv_ctx,
     size_t app_req_len(0);
 
     const_cast<wsrep_view_info_t&>(view_info).state_gap = st_required;
+
+    // We need to set the protocol version BEFORE the view callback, so that
+    // any version-dependent code is run using the correct version instead of -1.
+    if (view_info.view >= 0) // Primary configuration
+        establish_protocol_versions (repl_proto);
+
     wsrep_cb_status_t const rcode(
         view_cb_(app_ctx_, recv_ctx, &view_info, 0, 0, &app_req, &app_req_len));
 
@@ -1578,8 +1630,6 @@ galera::ReplicatorSMM::process_conf_change(void*                    recv_ctx,
 
     if (view_info.view >= 0) // Primary configuration
     {
-        establish_protocol_versions (repl_proto);
-
         // we have to reset cert initial position here, SST does not contain
         // cert index yet (see #197).
         // Also this must be done before releasing GCache buffers.
@@ -1919,6 +1969,7 @@ wsrep_status_t galera::ReplicatorSMM::cert(TrxHandle* trx)
             }
             break;
         case Certification::TEST_FAILED:
+#if 0
             if (gu_unlikely(trx->is_toi() && applicable)) // small sanity check
             {
                 // In some rare scenarios (e.g., when we have multiple
@@ -1940,6 +1991,19 @@ wsrep_status_t galera::ReplicatorSMM::cert(TrxHandle* trx)
                 local_monitor_.leave(lo);
                 abort();
             }
+#endif
+            /* Code above fails to handle TOI (read DDL) transaction
+            as DDL are non-atomic and so can't be rolled back in case of
+            certification failure. But given the TOI flow, certification
+            checks are done well before the real-action starts and so
+            error returned at stage shouldn't cause any rollback for TOI/DDL. */
+            if (gu_unlikely(trx->is_toi() && applicable))
+                log_info << "Certification failed for TO isolated action: "
+                          << *trx;
+            else
+                log_debug << "Certification failed for replicated action: "
+                          << *trx;
+
             local_cert_failures_ += trx->is_local();
             trx->set_state(TrxHandle::S_MUST_ABORT);
             retval = WSREP_TRX_FAIL;
