@@ -118,6 +118,7 @@ namespace galera
             TrxHandle* trx = reinterpret_cast<TrxHandle*>(trx_handle);
             CommitOrder co(*trx, co_mode_);
             commit_monitor_.leave(co);
+            GU_DBUG_SYNC_WAIT("sync.applier_interim_commit.after_commit_leave");
             trx->mark_interim_committed(true);
             return WSREP_OK;
         }
@@ -129,6 +130,7 @@ namespace galera
             {
                 CommitOrder co(*trx, co_mode_);
                 commit_monitor_.leave(co);
+                GU_DBUG_SYNC_WAIT("sync.applier_post_commit.after_commit_leave");
             }
             trx->mark_interim_committed(false);
             return WSREP_OK;
@@ -394,9 +396,11 @@ namespace galera
                         << "commit order condition called in bypass mode";
                 case OOOC:
                     return true;
+                    // fall through
                 case LOCAL_OOOC:
                     if (trx_.is_local()) { return true; }
                     // in case of remote trx fall through
+                    // fall through
                 case NO_OOOC:
                     return (last_left + 1 == trx_.global_seqno());
                 }
@@ -557,17 +561,18 @@ namespace galera
 
         static int const       MAX_PROTO_VER;
         /*
-         * |------------------------------------------------------
-         * | protocol_version_ |  trx  version  | str_proto_ver_ |
-         * |------------------------------------------------------
-         * |                 1 |              1 |              0 |
-         * |                 2 |              1 |              1 |
-         * |                 3 |              2 |              1 |
-         * |                 4 |              2 |              1 |
-         * |                 5 |              3 |              1 |
-         * |                 6 |              3 |              2 |
-         * |                 7 |              3 |              2 |
-         * -------------------------------------------------------
+         * |--------------------------------------------------------------------|
+         * | protocol_version_ | trx version | str_proto_ver_ | record_set_ver_ |
+         * |--------------------------------------------------------------------|
+         * |                 1 |           1 |              0 |               1 |
+         * |                 2 |           1 |              1 |               1 |
+         * |                 3 |           2 |              1 |               1 |
+         * |                 4 |           2 |              1 |               1 |
+         * |                 5 |           3 |              1 |               1 |
+         * |                 6 |           3 |              2 |               1 |
+         * |                 7 |           3 |              2 |               1 |
+         * |                 8 |           4 |              2 |               2 |
+         * |--------------------------------------------------------------------|
          */
 
         int                    str_proto_ver_;// state transfer request protocol
