@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2010-2014 Codership Oy <info@codership.com>
+// Copyright (C) 2010-2017 Codership Oy <info@codership.com>
 //
 
 
@@ -21,6 +21,7 @@
 #include "gu_utils.hpp"
 #include "gu_macros.hpp"
 #include "gu_mem_pool.hpp"
+#include "gu_limits.h" // page size stuff
 
 #include <set>
 
@@ -36,18 +37,32 @@ namespace galera
     public:
 
         /* signed int here is to detect SIZE < sizeof(TrxHandle) */
-        static int const LOCAL_STORAGE_SIZE = GU_PAGE_SIZE * 2; // 8K
+        static size_t LOCAL_STORAGE_SIZE()
+        {
+            static size_t const ret(gu_page_size_multiple(1 << 13 /* 8Kb */));
+            return ret;
+        }
 
         struct Params
         {
-            std::string     working_dir_;
-            int             version_;
-            KeySet::Version key_format_;
-            int             max_write_set_size_;
-            Params (const std::string& wdir, int ver, KeySet::Version kformat,
-                    int max_write_set_size = WriteSetNG::MAX_SIZE) :
-                working_dir_(wdir), version_(ver), key_format_(kformat),
-                max_write_set_size_(max_write_set_size) {}
+            std::string            working_dir_;
+            int                    version_;
+            KeySet::Version        key_format_;
+            gu::RecordSet::Version record_set_ver_;
+            int                    max_write_set_size_;
+
+            Params (const std::string& wdir,
+                    int                ver,
+                    KeySet::Version    kformat,
+                    gu::RecordSet::Version rsv = gu::RecordSet::VER2,
+                    int                max_write_set_size = WriteSetNG::MAX_SIZE)
+                :
+                working_dir_       (wdir),
+                version_           (ver),
+                key_format_        (kformat),
+                record_set_ver_    (rsv),
+                max_write_set_size_(max_write_set_size)
+            {}
         };
 
         static const Params Defaults;
@@ -267,6 +282,9 @@ namespace galera
         bool is_committed() const { return committed_; }
         void mark_committed() { committed_ = true; }
 
+        bool is_interim_committed() const { return interim_committed_; }
+        void mark_interim_committed(bool val) { interim_committed_ = val; }
+
         void set_received (const void*   action,
                            wsrep_seqno_t seqno_l,
                            wsrep_seqno_t seqno_g)
@@ -290,6 +308,18 @@ namespace galera
             }
         }
 
+        /* obtain global and depends seqno from the writeset (IST) */
+        void set_received_from_ws()
+        {
+            wsrep_seqno_t const seqno_g(write_set_in_.seqno());
+            set_received(0, -1, seqno_g);
+            wsrep_seqno_t const seqno_d
+                (std::max<wsrep_seqno_t>
+                    (global_seqno_ - write_set_in_.pa_range(),
+                     WSREP_SEQNO_UNDEFINED));
+            set_depends_seqno(seqno_d);
+        }
+
         void set_last_seen_seqno(wsrep_seqno_t last_seen_seqno)
         {
             assert (last_seen_seqno >= 0);
@@ -541,7 +571,10 @@ namespace galera
             assert(wso_);
             return *reinterpret_cast<WriteSetOut*>(this + 1);
         }
-        const WriteSetOut& write_set_out() const { return write_set_out(); }
+        const WriteSetOut& write_set_out() const
+        {
+            return const_cast<TrxHandle*>(this)->write_set_out();
+        }
 
         const WriteSetIn&  write_set_in () const { return write_set_in_;  }
 
@@ -646,6 +679,7 @@ namespace galera
             local_             (false),
             certified_         (false),
             committed_         (false),
+            interim_committed_ (false),
             exit_loop_         (false),
             wso_               (false),
             mac_               ()
@@ -689,6 +723,7 @@ namespace galera
             local_             (true),
             certified_         (false),
             committed_         (false),
+            interim_committed_ (false),
             exit_loop_         (false),
             wso_               (new_version()),
             mac_               ()
@@ -710,12 +745,14 @@ namespace galera
 
                 WriteSetOut* wso = &write_set_out();
                 assert(static_cast<void*>(wso) == static_cast<void*>(store));
+                assert((uintptr_t(wso) % GU_WORD_BYTES) == 0);
 
                 new (wso) WriteSetOut (params.working_dir_,
                                        trx_id_, params.key_format_,
                                        store      + sizeof(WriteSetOut),
                                        store_size - sizeof(WriteSetOut),
                                        0,
+                                       params.record_set_ver_,
                                        WriteSetNG::MAX_VERSION,
                                        DataSet::MAX_VERSION,
                                        DataSet::MAX_VERSION,
@@ -758,6 +795,7 @@ namespace galera
         bool                   local_;
         bool                   certified_;
         bool                   committed_;
+        bool                   interim_committed_;
         bool                   exit_loop_;
         bool                   wso_;
         Mac                    mac_;
