@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2013 Codership Oy <info@codership.com>
+// Copyright (C) 2013-2017 Codership Oy <info@codership.com>
 //
 
 /*
@@ -25,7 +25,7 @@
 #include <string>
 #include <iomanip>
 
-#include <pthread.h>
+#include <gu_threads.h>
 
 namespace galera
 {
@@ -130,7 +130,12 @@ namespace galera
             {
                 switch (ver)
                 {
-                case VER3: return V3_SIZE;
+                case VER3:
+                {
+                    GU_COMPILE_ASSERT(0 == (V3_SIZE % GU_MIN_ALIGNMENT),
+                                      unaligned_header_size);
+                    return V3_SIZE;
+                }
                 }
 
                 log_fatal << "Unknown writeset version: " << ver;
@@ -143,6 +148,7 @@ namespace galera
             Header (Version ver)
             : local_(), ptr_(local_), ver_(ver), size_(size(ver)), chksm_()
             {
+                assert((uintptr_t(ptr_) % GU_WORD_BYTES) == 0);
                 assert (size_t(size_) <= sizeof(local_));
             }
 
@@ -175,12 +181,13 @@ namespace galera
             Header (const gu::Buf& buf)
                 :
                 local_(),
-                ptr_  (reinterpret_cast<gu::byte_t*>(
-                           const_cast<void*>(buf.ptr))),
+                ptr_  (static_cast<gu::byte_t*>(const_cast<void*>(buf.ptr))),
                 ver_  (version(buf)),
                 size_ (check_size(ver_, ptr_, buf.size)),
                 chksm_(ver_, ptr_, size_)
-            {}
+            {
+                assert((uintptr_t(ptr_) % GU_WORD_BYTES) == 0);
+            }
 
             Header () : local_(), ptr_(NULL), ver_(), size_(0), chksm_()
             {}
@@ -189,8 +196,7 @@ namespace galera
             void read_buf (const gu::Buf& buf)
             {
                 ver_ = version(buf);
-                ptr_ =
-                    reinterpret_cast<gu::byte_t*>(const_cast<void*>(buf.ptr));
+                ptr_ = static_cast<gu::byte_t*>(const_cast<void*>(buf.ptr));
                 size_ = check_size (ver_, ptr_, buf.size);
                 Checksum::verify(ver_, ptr_, size_);
             }
@@ -292,16 +298,6 @@ namespace galera
                 return ptr_ + size();
             }
 
-            uint64_t get_checksum() const
-            {
-                const void* const checksum_ptr
-                    (reinterpret_cast<const gu::byte_t*>(ptr_) + size_ -
-                     V3_CHECKSUM_SIZE);
-
-                return gu::gtoh<Checksum::type_t>(
-                    *(static_cast<const Checksum::type_t*>(checksum_ptr)));
-            }
-
             /* to set seqno and parallel applying range after certification */
             void set_seqno(const wsrep_seqno_t& seqno, uint16_t pa_range);
 
@@ -335,12 +331,10 @@ namespace galera
             public:
                 typedef uint64_t type_t;
 
-                /* produce value (corrected for endianness) */
                 static void
                 compute (const void* ptr, size_t size, type_t& value)
                 {
                     gu::FastHash::digest (ptr, size, value);
-                    value = gu::htog<type_t>(value);
                 }
 
                 static void
@@ -431,7 +425,7 @@ namespace galera
             {
                 Checksum::type_t cval;
                 Checksum::compute (ptr, size, cval);
-                *reinterpret_cast<Checksum::type_t*>(ptr + size) = cval;
+                gu::serialize(cval, ptr, size);
             }
         }; /* class Header */
 
@@ -491,6 +485,7 @@ namespace galera
                      gu::byte_t*             reserved,
                      size_t                  reserved_size,
                      uint16_t                flags    = 0,
+                     gu::RecordSet::Version  rsv      = gu::RecordSet::VER2,
                      WriteSetNG::Version     ver      = WriteSetNG::MAX_VERSION,
                      DataSet::Version        dver     = DataSet::MAX_VERSION,
                      DataSet::Version        uver     = DataSet::MAX_VERSION,
@@ -502,20 +497,22 @@ namespace galera
             kbn_   (base_name_),
             keys_  (reserved,
                     (reserved_size >>= 6, reserved_size <<= 3, reserved_size),
-                    kbn_, kver),
+                    kbn_, kver, rsv),
             /* 5/8 of reserved goes to data set  */
             dbn_   (base_name_),
-            data_  (reserved + reserved_size, reserved_size*5, dbn_, dver),
+            data_  (reserved + reserved_size, reserved_size*5, dbn_, dver, rsv),
             /* 2/8 of reserved goes to unordered set  */
             ubn_   (base_name_),
-            unrd_  (reserved + reserved_size*6, reserved_size*2, ubn_, uver),
+            unrd_  (reserved + reserved_size*6, reserved_size*2, ubn_, uver,rsv),
             /* annotation set is not allocated unless requested */
             abn_   (base_name_),
             annt_  (NULL),
             left_  (max_size - keys_.size() - data_.size() - unrd_.size()
                     - header_.size()),
             flags_ (flags)
-        {}
+        {
+            assert ((uintptr_t(reserved) % GU_WORD_BYTES) == 0);
+        }
 
         ~WriteSetOut() { delete annt_; }
 
@@ -538,7 +535,9 @@ namespace galera
         {
             if (NULL == annt_)
             {
-                annt_ = new DataSetOut(NULL, 0, abn_, DataSet::MAX_VERSION);
+                annt_ = new DataSetOut(NULL, 0, abn_, DataSet::MAX_VERSION,
+                                       // use the same version as the dataset
+                                       data_.gu::RecordSet::version());
                 left_ -= annt_->size();
             }
 
@@ -683,7 +682,7 @@ namespace galera
               check_thr_(false),
               check_ (false)
         {
-            init (st);
+            gu_trace(init(st));
         }
 
         WriteSetIn ()
@@ -706,7 +705,7 @@ namespace galera
 
             header_.read_buf (buf);
             size_ = buf.size;
-            init (st);
+            gu_trace(init(st));
         }
 
         void read_buf (const gu::byte_t* const ptr, ssize_t const len)
@@ -722,7 +721,7 @@ namespace galera
             if (gu_unlikely(check_thr_))
             {
                 /* checksum was performed in a parallel thread */
-                pthread_join (check_thr_id_, NULL);
+                gu_thread_join (check_thr_id_, NULL);
             }
 
             delete annt_;
@@ -758,9 +757,9 @@ namespace galera
             if (gu_unlikely(check_thr_))
             {
                 /* checksum was performed in a parallel thread */
-                pthread_join (check_thr_id_, NULL);
+                gu_thread_join (check_thr_id_, NULL);
                 check_thr_ = false;
-                checksum_fin();
+                gu_trace(checksum_fin());
             }
         }
 
@@ -798,7 +797,7 @@ namespace galera
         DataSetIn          data_;
         DataSetIn          unrd_;
         DataSetIn*         annt_;
-        pthread_t          check_thr_id_;
+        gu_thread_t        check_thr_id_;
         bool mutable       check_thr_;
         bool               check_;
 
