@@ -339,7 +339,7 @@ void gcomm::GMCast::close(bool force)
 {
     log_debug << "gmcast " << uuid() << " close";
     pstack_.pop_proto(this);
-    if (mcast_ != 0)
+    if (mcast_)
     {
         mcast_->close();
         // delete mcast;
@@ -543,7 +543,7 @@ void gcomm::GMCast::handle_connected(Proto* rp)
     log_debug << "transport " << tp << " connected";
     if (rp->state() == Proto::S_INIT)
     {
-        log_debug << "sending hanshake";
+        log_debug << "sending handshake";
         // accepted socket was waiting for underlying transport
         // handshake to finish
         rp->send_handshake();
@@ -619,7 +619,7 @@ void gcomm::GMCast::handle_established(Proto* est)
         i = remote_addrs_.insert_unique(
             make_pair(est->remote_addr(),
                       AddrEntry(gu::datetime::Date::now(),
-                                gu::datetime::Date::max(),
+                                gu::datetime::Date::now(),
                                 est->remote_uuid())));
     }
 
@@ -905,7 +905,7 @@ void gcomm::GMCast::update_addresses()
 
     Segment& local_segment(segment_map_[segment_]);
 
-    if (mcast_ != 0)
+    if (mcast_)
     {
         log_debug << mcast_addr_;
         local_segment.push_back(mcast_.get());
@@ -983,7 +983,6 @@ void gcomm::GMCast::reconnect()
         }
     }
 
-
     for (i = remote_addrs_.begin(); i != remote_addrs_.end(); i = i_next)
     {
         i_next = i, ++i_next;
@@ -1086,11 +1085,16 @@ void gcomm::GMCast::check_liveness()
             p->state() < Proto::S_FAILED &&
             p->tstamp() + peer_timeout_ < now)
         {
-            log_info << self_string()
-                     << " connection to peer "
-                     << p->remote_uuid() << " with addr "
-                     << p->remote_addr()
-                     << " timed out, no messages seen in " << peer_timeout_;
+            // Only log if addr has not been blacklisted
+            if (addr_blacklist_.find(p->remote_addr()) == addr_blacklist_.end())
+            {
+                log_info << self_string()
+                         << " connection to peer "
+                         << p->remote_uuid() << " with addr "
+                         << p->remote_addr()
+                         << " timed out, no messages seen in " << peer_timeout_
+                         << " (gmcast.peer_timeout)";
+            }
             p->set_state(Proto::S_FAILED);
             handle_failed(p);
         }
@@ -1297,7 +1301,7 @@ void gcomm::GMCast::handle_up(const void*        id,
     {
         gmcast_accept();
     }
-    else if (mcast_.get() != 0 && id == mcast_->id())
+    else if (mcast_ && id == mcast_->id())
     {
         Message msg;
 
@@ -1323,7 +1327,7 @@ void gcomm::GMCast::handle_up(const void*        id,
             return;
         }
 
-        if (msg.type() >= Message::T_USER_BASE)
+        if (msg.type() >= Message::GMCAST_T_USER_BASE)
         {
             gu_trace(send_up(Datagram(dg, dg.offset() + msg.serial_size()),
                              ProtoUpMeta(msg.source_uuid())));
@@ -1365,7 +1369,7 @@ void gcomm::GMCast::handle_up(const void*        id,
                 return;
             }
 
-            if (msg.type() >= Message::T_USER_BASE)
+            if (msg.type() >= Message::GMCAST_T_USER_BASE)
             {
                 if (evict_list().empty() == false &&
                     evict_list().find(msg.source_uuid()) != evict_list().end())
@@ -1450,7 +1454,7 @@ void gcomm::GMCast::handle_up(const void*        id,
 
 int gcomm::GMCast::handle_down(Datagram& dg, const ProtoDownMeta& dm)
 {
-    Message msg(version_, Message::T_USER_BASE, uuid(), 1, segment_);
+    Message msg(version_, Message::GMCAST_T_USER_BASE, uuid(), 1, segment_);
 
     // handle relay set first, skip these peers below
     if (relay_set_.empty() == false)
