@@ -552,8 +552,9 @@ gcs_group_handle_uuid_msg  (gcs_group_t* group, const gcs_recv_msg_t* msg)
 
     if (GCS_GROUP_WAIT_STATE_UUID == group->state &&
         0 == msg->sender_idx /* check that it is from the representative */) {
-        group->state_uuid = *(gu_uuid_t*)msg->buf;
-        group->state      = GCS_GROUP_WAIT_STATE_MSG;
+//        group->state_uuid = *(gu_uuid_t*)msg->buf;
+        gu_uuid_copy(&group->state_uuid, (const gu_uuid_t*)msg->buf);
+        group->state = GCS_GROUP_WAIT_STATE_MSG;
     }
     else {
         gu_warn ("Stray state UUID msg: " GU_UUID_FORMAT
@@ -1190,7 +1191,8 @@ group_find_ist_donor (const gcs_group_t* const group,
 
     if (ist_seqno < safe_ist_seqno) {
         // unsafe to perform ist.
-        gu_debug("fallback to sst. ist_seqno < safe_ist_seqno");
+        gu_info("may fallback to sst. ist_seqno [%lld] < safe_ist_seqno [%lld]",
+                (long long) ist_seqno, (long long) safe_ist_seqno);
         return -1;
     }
 
