@@ -74,9 +74,6 @@ static char *audit_log_include_commands= NULL;
 PSI_memory_key key_memory_audit_log_logger_handle;
 PSI_memory_key key_memory_audit_log_handler;
 PSI_memory_key key_memory_audit_log_buffer;
-PSI_memory_key key_memory_audit_log_record_buffer;
-PSI_memory_key key_memory_audit_log_query_stack;
-PSI_memory_key key_memory_audit_log_thd_local;
 PSI_memory_key key_memory_audit_log_accounts;
 PSI_memory_key key_memory_audit_log_databases;
 PSI_memory_key key_memory_audit_log_commands;
@@ -86,9 +83,6 @@ static PSI_memory_info all_audit_log_memory[]=
   {&key_memory_audit_log_logger_handle, "audit_log_logger_handle", 0},
   {&key_memory_audit_log_handler, "audit_log_handler", 0},
   {&key_memory_audit_log_buffer, "audit_log_buffer", 0},
-  {&key_memory_audit_log_record_buffer, "audit_log_record_buffer", 0},
-  {&key_memory_audit_log_query_stack, "audit_log_query_stack", 0},
-  {&key_memory_audit_log_thd_local, "audit_log_thd_local", 0},
   {&key_memory_audit_log_accounts, "audit_log_accounts", 0},
   {&key_memory_audit_log_databases, "audit_log_databases", 0},
   {&key_memory_audit_log_commands, "audit_log_commands", 0},
@@ -144,6 +138,8 @@ ulonglong next_record_id()
 #define MAX_RECORD_ID_SIZE  50
 #define MAX_TIMESTAMP_SIZE  25
 
+void plugin_thdvar_safe_update(MYSQL_THD thd, struct st_mysql_sys_var *var,
+                               char **dest, const char *value);
 
 static
 char *make_timestamp(char *buf, size_t buf_len, time_t t)
@@ -180,37 +176,48 @@ typedef struct
 
 static
 void escape_buf(const char *in, size_t *inlen, char *out, size_t *outlen,
-                const escape_rule_t *escape_rules)
+                const escape_rule_t *control_escape_rules,
+                const escape_rule_t *other_escape_rules)
 {
   char* outstart = out;
   const char* base = in;
   char* outend = out + *outlen;
   const char* inend;
-  const escape_rule_t *rule;
-  my_bool replaced;
+  const escape_rule_t *replace_rule = NULL;
 
   inend = in + (*inlen);
 
   while ((in < inend) && (out < outend))
   {
-    replaced= FALSE;
-    for (rule= escape_rules; rule->character; rule++)
+    replace_rule = NULL;
+    if ((unsigned char)(*in) < 32) {
+      if (control_escape_rules[(unsigned int)*in].character) {
+        replace_rule = &control_escape_rules[(unsigned int)*in];
+      }
+    } else
     {
-      if (*in == rule->character)
+      const escape_rule_t *rule = NULL;
+      for (rule= other_escape_rules; rule->character; rule++)
       {
-        if ((outend - out) < (int) rule->length)
-          goto end_of_buffer;
-        memcpy(out, rule->replacement, rule->length);
-        out += rule->length;
-        replaced= TRUE;
-        break;
+        if (*in == rule->character)
+        {
+          replace_rule = rule;
+          break;
+        }
       }
     }
-    if (!replaced)
+    if (replace_rule)
+    {
+          if ((outend - out) < (ptrdiff_t) replace_rule->length)
+            break;
+          memcpy(out, replace_rule->replacement, replace_rule->length);
+          out += replace_rule->length;
+    } else
+    {
       *out++ = *in;
+    }
     ++in;
   }
-end_of_buffer:
   *outlen = out - outstart;
   *inlen = in - base;
 }
@@ -218,45 +225,153 @@ end_of_buffer:
 static
 void xml_escape(const char *in, size_t *inlen, char *out, size_t *outlen)
 {
-  const escape_rule_t rules[]=
+  // Most control sequences aren't supported before XML 1.1, and most
+  // tools only support 1.0. Our output is 1.0. Escaping them wouldn't make
+  // the output more valid.
+  static const escape_rule_t control_rules[]=
+  {
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { '\t', 5, "&#9;" },
+    { '\n', 6, "&#10;" },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { '\r', 6, "&#13;" },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+  };
+  static const escape_rule_t other_rules[]=
   {
     { '<',  4, "&lt;" },
     { '>',  4, "&gt;" },
     { '&',  5, "&amp;" },
-    { '\r', 5, "&#13;" },
-    { '\n', 5, "&#10;" },
     { '"',  6, "&quot;" },
     { 0,  0, NULL }
   };
 
-  escape_buf(in, inlen, out, outlen, rules);
+  escape_buf(in, inlen, out, outlen, control_rules, other_rules);
 }
 
 static
 void json_escape(const char *in, size_t *inlen, char *out, size_t *outlen)
 {
-  const escape_rule_t rules[]=
+  static const escape_rule_t control_rules[]=
+  {
+    { 0,  6, "\\u0000" },
+    { 1,  6, "\\u0001" },
+    { 2,  6, "\\u0002" },
+    { 3,  6, "\\u0003" },
+    { 4,  6, "\\u0004" },
+    { 5,  6, "\\u0005" },
+    { 6,  6, "\\u0006" },
+    { 7,  6, "\\u0007" },
+    { '\b',  2, "\\b" },
+    { '\t',  2, "\\t" },
+    { '\n',  2, "\\n" },
+    { 11, 6, "\\u000B" },
+    { '\f',  2, "\\f" },
+    { '\r',  2, "\\r" },
+    { 14, 6, "\\u000E" },
+    { 15, 6, "\\u000F" },
+    { 16, 6, "\\u0010" },
+    { 17, 6, "\\u0011" },
+    { 18, 6, "\\u0012" },
+    { 19, 6, "\\u0013" },
+    { 20, 6, "\\u0014" },
+    { 21, 6, "\\u0015" },
+    { 22, 6, "\\u0016" },
+    { 23, 6, "\\u0017" },
+    { 24, 6, "\\u0018" },
+    { 25, 6, "\\u0019" },
+    { 26, 6, "\\u001A" },
+    { 27, 6, "\\u001B" },
+    { 28, 6, "\\u001C" },
+    { 29, 6, "\\u001D" },
+    { 30, 6, "\\u001E" },
+    { 31, 6, "\\u001F" },
+  };
+
+  static const escape_rule_t other_rules[]=
   {
     { '\\', 2, "\\\\" },
     { '"',  2, "\\\"" },
-    { '\r',  2, "\\r" },
-    { '\n',  2, "\\n" },
+    { '/',  2, "\\/" },
     { 0,  0, NULL }
   };
 
-  escape_buf(in, inlen, out, outlen, rules);
+  escape_buf(in, inlen, out, outlen, control_rules, other_rules);
 }
 
 static
 void csv_escape(const char *in, size_t *inlen, char *out, size_t *outlen)
 {
-  const escape_rule_t rules[]=
+  // We do not have any standard control escape rules for CSVs
+  static const escape_rule_t control_rules[]=
+  {
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+    { 0,  0, NULL },
+  };
+
+  static const escape_rule_t other_rules[]=
   {
     { '"',  2, "\"\"" },
     { 0,  0, NULL }
   };
 
-  escape_buf(in, inlen, out, outlen, rules);
+  escape_buf(in, inlen, out, outlen, control_rules, other_rules);
 }
 
 static const escape_buf_func_t format_escape_func[]=
@@ -804,8 +919,10 @@ typedef struct
  */
 typedef struct
 {
-  /* size of allocated large buffer to for record formatting */
+  /* size of allocated large buffer for record formatting */
   size_t record_buffer_size;
+  /* large buffer for record formatting */
+  char *record_buffer;
   /* skip session logging */
   my_bool skip_session;
   /* skip logging for the next query */
@@ -850,15 +967,91 @@ int audit_log_plugin_init(MYSQL_PLUGIN plugin_info)
   mysql_memory_register(AUDIT_LOG_PSI_CATEGORY, all_audit_log_memory, count);
   logger_init_mutexes();
 
+  audit_log_filter_init();
+
+  if (audit_log_exclude_accounts != NULL && audit_log_include_accounts != NULL)
+  {
+    my_plugin_log_message(&plugin_ptr, MY_ERROR_LEVEL,
+                          "Both 'audit_log_exclude_accounts' and "
+                          "'audit_log_include_accounts' are not NULL\n");
+    goto validation_error;
+  }
+
+  if (audit_log_exclude_commands != NULL && audit_log_include_commands != NULL)
+  {
+    my_plugin_log_message(&plugin_ptr, MY_ERROR_LEVEL,
+                          "Both 'audit_log_exclude_commands' and "
+                          "'audit_log_include_commands' are not NULL\n");
+    goto validation_error;
+  }
+
+  if (audit_log_exclude_databases != NULL
+      && audit_log_include_databases != NULL)
+  {
+    my_plugin_log_message(&plugin_ptr, MY_ERROR_LEVEL,
+                          "Both 'audit_log_exclude_databases' and "
+                          "'audit_log_include_databases' are not NULL\n");
+    goto validation_error;
+  }
+
+  if (audit_log_exclude_accounts != NULL)
+  {
+    audit_log_exclude_accounts= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_exclude_accounts,
+                                          MYF(MY_FAE));
+    audit_log_set_exclude_accounts(audit_log_exclude_accounts);
+  }
+  if (audit_log_include_accounts != NULL)
+  {
+    audit_log_include_accounts= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_include_accounts,
+                                          MYF(MY_FAE));
+    audit_log_set_include_accounts(audit_log_include_accounts);
+  }
+  if (audit_log_exclude_commands != NULL)
+  {
+    audit_log_exclude_commands= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_exclude_commands,
+                                          MYF(MY_FAE));
+    audit_log_set_exclude_commands(audit_log_exclude_commands);
+  }
+  if (audit_log_include_commands != NULL)
+  {
+    audit_log_include_commands= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_include_commands,
+                                          MYF(MY_FAE));
+    audit_log_set_include_commands(audit_log_include_commands);
+  }
+  if (audit_log_exclude_databases != NULL)
+  {
+    audit_log_exclude_databases= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_exclude_databases,
+                                          MYF(MY_FAE));
+    audit_log_set_exclude_databases(audit_log_exclude_databases);
+  }
+  if (audit_log_include_databases != NULL)
+  {
+    audit_log_include_databases= my_strdup(PSI_NOT_INSTRUMENTED,
+                                          audit_log_include_databases,
+                                          MYF(MY_FAE));
+    audit_log_set_include_databases(audit_log_include_databases);
+  }
+
   if (init_new_log_file())
     return(1);
 
   if (audit_log_audit_record(buf, sizeof(buf), "Audit", time(NULL), &len))
     audit_log_write(buf, len);
 
-  audit_log_filter_init();
+  return 0;
 
-  return(0);
+validation_error:
+
+  audit_log_exclude_accounts= audit_log_include_accounts= NULL;
+  audit_log_exclude_commands= audit_log_include_commands= NULL;
+  audit_log_exclude_databases= audit_log_include_databases= NULL;
+
+  return 1;
 }
 
 
@@ -1719,9 +1912,13 @@ static MYSQL_SYSVAR_STR(include_commands, audit_log_include_commands,
        audit_log_include_commands_update, NULL);
 
 static MYSQL_THDVAR_STR(local,
-                        PLUGIN_VAR_READONLY | PLUGIN_VAR_MEMALLOC | \
-                        PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT,
-                        "Local store.", NULL, NULL, "");
+       PLUGIN_VAR_READONLY | PLUGIN_VAR_MEMALLOC | \
+       PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT,
+       "Local store.", NULL, NULL, "");
+
+static MYSQL_THDVAR_ULONG(local_ptr,
+       PLUGIN_VAR_READONLY | PLUGIN_VAR_NOSYSVAR | PLUGIN_VAR_NOCMDOPT,
+       "Local store ptr.", NULL, NULL, 0, 0, ULONG_MAX, 0);
 
 static struct st_mysql_sys_var* audit_log_system_variables[] =
 {
@@ -1746,24 +1943,34 @@ static struct st_mysql_sys_var* audit_log_system_variables[] =
   MYSQL_SYSVAR(exclude_commands),
   MYSQL_SYSVAR(include_commands),
   MYSQL_SYSVAR(local),
+  MYSQL_SYSVAR(local_ptr),
   NULL
 };
 
+char thd_local_init_buf[sizeof(audit_log_thd_local)];
+
+void MY_ATTRIBUTE((constructor)) audit_log_so_init()
+{
+  memset(thd_local_init_buf, 1, sizeof(thd_local_init_buf) - 1);
+  thd_local_init_buf[sizeof(thd_local_init_buf) - 1]= 0;
+}
+
 /*
  Return pointer to THD specific data.
  */
 static
 audit_log_thd_local *get_thd_local(MYSQL_THD thd)
 {
-  audit_log_thd_local *local= (audit_log_thd_local *) THDVAR(thd, local);
+  audit_log_thd_local *local= (audit_log_thd_local *) THDVAR(thd, local_ptr);
+
+  compile_time_assert(sizeof(THDVAR(thd, local_ptr)) >= sizeof(void *));
 
   if (unlikely(local == NULL))
   {
-    local= (audit_log_thd_local *)
-            my_malloc(key_memory_audit_log_thd_local,
-                      sizeof(audit_log_thd_local),
-                      MYF(MY_FAE | MY_ZEROFILL));
-    THDVAR(thd, local)= (char *) local;
+    THDVAR_SET(thd, local, thd_local_init_buf);
+    local= (audit_log_thd_local *) THDVAR(thd, local);
+    memset(local, 0, sizeof(audit_log_thd_local));
+    THDVAR(thd, local_ptr)= (ulong) local;
 
     realloc_stack_frames(thd, 4);
   }
@@ -1778,14 +1985,22 @@ static
 char *get_record_buffer(MYSQL_THD thd, size_t size)
 {
   audit_log_thd_local *local= get_thd_local(thd);
-  char *buf= THDVAR(thd, record_buffer);
+  char *buf= local->record_buffer;
 
   if (local->record_buffer_size < size)
   {
     local->record_buffer_size= size;
-    buf= (char *) my_realloc(key_memory_audit_log_record_buffer, buf, size,
-                             MYF(MY_FAE | MY_ALLOW_ZERO_PTR));
-    THDVAR(thd, record_buffer)= (char *) buf;
+
+    buf = (char *) my_malloc(PSI_NOT_INSTRUMENTED, size, MYF(MY_FAE));
+    memset(buf, 1, size - 1);
+    buf[size - 1]= 0;
+
+    THDVAR_SET(thd, record_buffer, buf);
+
+    my_free(buf);
+
+    buf = (char *) THDVAR(thd, record_buffer);
+    local->record_buffer = buf;
   }
 
   return buf;
@@ -1803,15 +2018,24 @@ query_stack_frame *realloc_stack_frames(MYSQL_THD thd, size_t size)
 
   if (local->stack.size < size)
   {
-    local->stack.size= size;
-    stack= (query_stack_frame *) my_realloc(key_memory_audit_log_query_stack,
-                                            stack,
-                                            size * sizeof(query_stack_frame),
-                                            MYF(MY_FAE
-                                                | MY_ALLOW_ZERO_PTR
-                                                | MY_ZEROFILL));
+    char *buf= (char *) my_malloc(PSI_NOT_INSTRUMENTED,
+                                  (local->stack.size + size) *
+                                  sizeof(query_stack_frame),
+                                  MYF(MY_FAE));
+    memset(buf + local->stack.size * sizeof(query_stack_frame), 1,
+           size * sizeof(query_stack_frame) - 1);
+    buf[(local->stack.size + size) * sizeof(query_stack_frame) - 1]= 0;
+    if (local->stack.size > 0)
+      memcpy(buf, stack, local->stack.size * sizeof(query_stack_frame));
+    THDVAR_SET(thd, query_stack,
+               buf + local->stack.size * sizeof(query_stack_frame));
+    stack= (query_stack_frame *) THDVAR(thd, query_stack);
+    memset(stack, 0, size * sizeof(query_stack_frame));
+    if (local->stack.size > 0)
+      memcpy(stack, buf, local->stack.size * sizeof(query_stack_frame));
     local->stack.frames= stack;
-    THDVAR(thd, query_stack)= (char *) stack;
+    local->stack.size= size;
+    my_free(buf);
   }
 
   return stack;
