@@ -1,22 +1,15 @@
 /***********************************************************************
 
-Copyright (c) 2011, 2018, Oracle and/or its affiliates. All rights reserved.
+Copyright (c) 2011, 2017, Oracle and/or its affiliates. All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License, version 2.0,
-as published by the Free Software Foundation.
+This program is free software; you can redistribute it and/or modify it
+under the terms of the GNU General Public License as published by the
+Free Software Foundation; version 2 of the License.
 
-This program is also distributed with certain software (including
-but not limited to OpenSSL) that is licensed under separate terms,
-as designated in a particular file or component or in included license
-documentation.  The authors of MySQL hereby grant you an additional
-permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License, version 2.0, for more details.
+This program is distributed in the hope that it will be useful, but
+WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
+Public License for more details.
 
 You should have received a copy of the GNU General Public License along
 with this program; if not, write to the Free Software Foundation, Inc.,
@@ -1142,9 +1135,6 @@ have_conn:
 			}
 		}
 	} else {
-        bool auto_commit = (engine->read_batch_size == 1 &&
-                !(engine->cfg_status & IB_CFG_DISABLE_ROWLOCK)) ? true : false;
-
 		assert(conn_option == CONN_MODE_READ);
 
 		if (!read_crsr) {
@@ -1152,12 +1142,15 @@ have_conn:
 				/* This is read operation, start a trx
 				with "read_write" parameter set to false */
 				conn_data->crsr_trx = ib_cb_trx_begin(
-					engine->trx_level, false, auto_commit);
+					engine->trx_level, false,
+					engine->read_batch_size == 1);
 				trx_updated = true;
 			} else {
 				ib_cb_trx_start(conn_data->crsr_trx,
 						engine->trx_level,
-						false, auto_commit, NULL);
+						false,
+						engine->read_batch_size == 1,
+						NULL);
 			}
 
 			err = innodb_api_begin(
@@ -1190,7 +1183,8 @@ have_conn:
 			/* This is read operation, start a trx
 			with "read_write" parameter set to false */
 			conn_data->crsr_trx = ib_cb_trx_begin(
-				engine->trx_level, false, auto_commit);
+				engine->trx_level, false,
+				engine->read_batch_size == 1);
 
 			trx_updated = true;
 
@@ -1236,7 +1230,8 @@ have_conn:
 			with "read_write" parameter set to false */
 			ib_cb_trx_start(conn_data->crsr_trx,
 					engine->trx_level,
-					false, auto_commit,
+					false,
+					engine->read_batch_size == 1,
 					NULL);
 
 			ib_cb_cursor_stmt_begin(conn_data->read_crsr);
