@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2018, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.
@@ -29,6 +29,7 @@
 #include "tztime.h"
 #include "crypt_genhash_impl.h"         /* CRYPT_MAX_PASSWORD_SIZE */
 #include "sql_user_table.h"
+#include <set>
 
 #ifndef DBUG_OFF
 #define HASH_STRING_WITH_QUOTE \
@@ -402,6 +403,7 @@ err:
   @param what_to_set  User attributes
   @param is_privileged_user     Whether caller has CREATE_USER_ACL
                                 or UPDATE_ACL over mysql.*
+  @param cmd          Command information
 
   @retval 0 ok
   @retval 1 ERROR;
@@ -410,7 +412,8 @@ err:
 bool set_and_validate_user_attributes(THD *thd,
                                       LEX_USER *Str,
                                       ulong &what_to_set,
-                                      bool is_privileged_user)
+                                      bool is_privileged_user,
+                                      const char * cmd)
 {
   bool user_exists= false;
   ACL_USER *acl_user;
@@ -581,6 +584,17 @@ bool set_and_validate_user_attributes(THD *thd,
                                              inbuflen))
     {
       plugin_unlock(0, plugin);
+
+      /*
+        generate_authentication_string may return error status
+        without setting actual error.
+      */
+      if (!thd->is_error())
+      {
+        String error_user;
+        append_user(thd, &error_user, Str, FALSE, FALSE);
+        my_error(ER_CANNOT_USER, MYF(0), cmd, error_user.c_ptr_safe());
+      }
       return(1);
     }
     if (buflen)
@@ -667,6 +681,9 @@ bool change_password(THD *thd, const char *host, const char *user,
   size_t escaped_hash_str_len= 0;
   bool result= true, rollback_whole_statement= false;
   int ret;
+#ifdef WITH_WSREP
+  const LEX_CSTRING query_save = thd->query();
+#endif /* WITH_WSREP */
 
   DBUG_ENTER("change_password");
   DBUG_PRINT("enter",("host: '%s'  user: '%s'  new_password: '%s'",
@@ -676,20 +693,6 @@ bool change_password(THD *thd, const char *host, const char *user,
   if (check_change_password(thd, host, user, new_password, new_password_len))
     DBUG_RETURN(1);
 
-#ifdef WITH_WSREP
-  const LEX_CSTRING query_save = thd->query();
-  if (WSREP(thd) && !thd->wsrep_applier)
-  {
-    query_length= sprintf(buff, "SET PASSWORD FOR '%-.120s'@'%-.120s'='%-.120s'",
-                          user ? user : "",
-                          host ? host : "",
-                          new_password);
-    thd->set_query(buff, query_length);
-
-    WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, (char*)"user", NULL);
-  }
-#endif /* WITH_WSREP */
-
   tables.init_one_table("mysql", 5, "user", 4, "user", TL_WRITE);
 
 #ifdef HAVE_REPLICATION
@@ -709,19 +712,27 @@ bool change_password(THD *thd, const char *host, const char *user,
       DBUG_RETURN(0);
   }
 #endif
+
+#ifdef WITH_WSREP
+  if (WSREP(thd) && !thd->wsrep_applier)
+  {
+    query_length= sprintf(buff, "SET PASSWORD FOR '%-.120s'@'%-.120s'='%-.120s'",
+          user ? user : "",
+          host ? host : "",
+          new_password);
+    //thd->set_query(buff, query_length, system_charset_info);
+    thd->set_query(buff, query_length);
+
+    WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, (char*)"user", NULL);
+  }
+#endif /* WITH_WSREP */
+
   if (!(table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT)))
     DBUG_RETURN(1);
 
   if (table_intact.check(table, &mysql_user_table_def))
     DBUG_RETURN(1);
 
-  if (!table->key_info)
-  {
-    my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
-             table->s->table_name.str);
-    DBUG_RETURN(1);
-  }
-
   /*
     This statement will be replicated as a statement, even when using
     row-based replication.  The flag will be reset at the end of the
@@ -788,7 +799,8 @@ bool change_password(THD *thd, const char *host, const char *user,
       thd->slave_thread)
     combo->uses_identified_by_clause= false;
     
-  if (set_and_validate_user_attributes(thd, combo, what_to_set, true))
+  if (set_and_validate_user_attributes(thd, combo, what_to_set,
+                                       true, "SET PASSWORD"))
   {
     result= 1;
     mysql_mutex_unlock(&acl_cache->lock);
@@ -1415,7 +1427,7 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
   ulong what_to_update= 0;
   bool is_anonymous_user= false;
   bool rollback_whole_statement= false;
-  std::set<LEX_USER *> users_not_to_log;
+  std::set<LEX_USER *> extra_users;
   DBUG_ENTER("mysql_create_user");
 
   /*
@@ -1459,7 +1471,8 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
       result= TRUE;
       continue;
     }
-    if (set_and_validate_user_attributes(thd, user_name, what_to_update, true))
+    if (set_and_validate_user_attributes(thd, user_name, what_to_update,
+                                         true, "CREATE USER"))
     {
       result= TRUE;
       continue;
@@ -1497,12 +1510,12 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
                             warn_user.c_ptr_safe());
         try
         {
-          users_not_to_log.insert(tmp_user_name);
+          extra_users.insert(user_name);
         }
         catch (...) {}
         continue;
       }
-     else
+      else
       {
         append_user(thd, &wrong_users, user_name, wrong_users.length() > 0,
                     false);
@@ -1524,12 +1537,11 @@ bool mysql_create_user(THD *thd, List <LEX_USER> &list, bool if_not_exists)
       my_error(ER_CANNOT_USER, MYF(0), "CREATE USER", wrong_users.c_ptr_safe());
   }
 
-  if (some_users_created ||
-      (if_not_exists && users_not_to_log.size() < list.elements))
+  if (some_users_created || (if_not_exists && !thd->is_error()))
   {
     String *rlb= &thd->rewritten_query;
     rlb->mem_free();
-    mysql_rewrite_create_alter_user(thd, rlb, &users_not_to_log);
+    mysql_rewrite_create_alter_user(thd, rlb, &extra_users);
 
     int ret= commit_owned_gtid_by_partial_command(thd);
 
@@ -1858,7 +1870,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
   bool save_binlog_row_based;
   bool is_privileged_user= false;
   bool rollback_whole_statement= false;
-  std::set<LEX_USER *> users_not_to_log;
+  std::set<LEX_USER *> extra_users;
   Acl_table_intact table_intact;
 
   DBUG_ENTER("mysql_alter_user");
@@ -1887,19 +1899,25 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
       DBUG_RETURN(false);
   }
 #endif
+
+#ifdef WITH_WSREP
+  /*
+    Perform the TOI after the replication filter check to avoid
+    replicating commands that won't be applied locally (due to a filter).
+    WSREP_TO_ISOLATION_BEGIN
+  */
+  if (WSREP(thd) && wsrep_to_isolation_begin(thd, WSREP_MYSQL_DB, NULL, NULL))
+  {
+      DBUG_RETURN(true);
+  }
+#endif
+
   if (!(table= open_ltable(thd, &tables, TL_WRITE, MYSQL_LOCK_IGNORE_TIMEOUT)))
     DBUG_RETURN(true);
 
   if (table_intact.check(table, &mysql_user_table_def))
     DBUG_RETURN(true);
 
-  if (!table->key_info)
-  {
-    my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
-             table->s->table_name.str);
-    DBUG_RETURN(true);
-  }
-
   /*
     This statement will be replicated as a statement, even when using
     row-based replication.  The flag will be reset at the end of the
@@ -1918,6 +1936,14 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
     ACL_USER *acl_user;
     ulong what_to_alter= 0;
 
+    if (acl_is_utility_user(tmp_user_from->user.str, tmp_user_from->host.str,
+                            NULL))
+    {
+      result= true;
+      append_user(thd, &wrong_users, tmp_user_from, wrong_users.length() > 0);
+      continue;
+    }
+
     /* add the defaults where needed */
     if (!(user_from= get_current_user(thd, tmp_user_from)))
     {
@@ -1927,60 +1953,33 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
       continue;
     }
 
-    /* look up the user */
-    if (!(acl_user= find_acl_user(user_from->host.str,
-                                   user_from->user.str, TRUE)))
-    {
-      if (if_exists)
-      {
-        String warn_user;
-        append_user(thd, &warn_user, user_from, FALSE, FALSE);
-        push_warning_printf(thd, Sql_condition::SL_NOTE,
-          ER_USER_DOES_NOT_EXIST,
-          ER_THD(thd, ER_USER_DOES_NOT_EXIST),
-          warn_user.c_ptr_safe());
-        try
-        {
-          users_not_to_log.insert(tmp_user_from);
-        }
-        catch (...) {}
-      }
-      else
-      {
-        result= TRUE;
-        append_user(thd, &wrong_users, user_from, wrong_users.length() > 0,
-          false);
-      }
-
-      continue;
-    }
-
     if (user_from && user_from->plugin.str)
       optimize_plugin_compare_by_pointer(&user_from->plugin);
 
     /* copy password expire attributes to individual lex user */
     user_from->alter_status= thd->lex->alter_password;
 
+    if (set_and_validate_user_attributes(thd, user_from, what_to_alter,
+                                         is_privileged_user, "ALTER USER"))
+    {
+      result= true;
+      continue;
+    }
+
     /*
       Check if the user's authentication method supports expiration only
       if PASSWORD EXPIRE attribute is specified
     */
     if (user_from->alter_status.update_password_expired_column &&
-        !auth_plugin_supports_expiration(acl_user->plugin.str))
+        !auth_plugin_supports_expiration(user_from->plugin.str))
     {
       result= true;
       append_user(thd, &wrong_users, user_from, wrong_users.length() > 0,
                   false);
       continue;
     }
-    if (set_and_validate_user_attributes(thd, user_from, what_to_alter,
-                                         is_privileged_user))
-    {
-      result= true;
-      continue;
-    }
 
-    if (!acl_user->user && 
+    if (!strcmp(user_from->user.str, "") &&
         (what_to_alter & PASSWORD_EXPIRE_ATTR) &&
         user_from->alter_status.update_password_expired_column)
     {
@@ -1991,6 +1990,34 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
       continue;
     }
 
+    /* look up the user */
+    if (!(acl_user= find_acl_user(user_from->host.str,
+                                  user_from->user.str, TRUE)))
+    {
+      if (if_exists)
+      {
+        String warn_user;
+        append_user(thd, &warn_user, user_from, FALSE, FALSE);
+        push_warning_printf(thd, Sql_condition::SL_NOTE,
+                            ER_USER_DOES_NOT_EXIST,
+                            ER_THD(thd, ER_USER_DOES_NOT_EXIST),
+                            warn_user.c_ptr_safe());
+        try
+        {
+          extra_users.insert(user_from);
+        }
+        catch (...) {}
+      }
+      else
+      {
+        result= TRUE;
+        append_user(thd, &wrong_users, user_from, wrong_users.length() > 0,
+                    false);
+      }
+
+      continue;
+    }
+
     /* update the mysql.user table */
     int ret= replace_user_table(thd, table, user_from, 0, false, true,
                                 what_to_alter);
@@ -2022,13 +2049,12 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
       my_error(ER_CANNOT_USER, MYF(0), "ALTER USER", wrong_users.c_ptr_safe());
   }
 
-  if (some_user_altered ||
-      (if_exists && users_not_to_log.size() < list.elements))
+  if (some_user_altered || (if_exists && !thd->is_error()))
   {
     /* do query rewrite for ALTER USER */
     String *rlb= &thd->rewritten_query;
     rlb->mem_free();
-    mysql_rewrite_create_alter_user(thd, rlb, &users_not_to_log);
+    mysql_rewrite_create_alter_user(thd, rlb, &extra_users);
 
     int ret= commit_owned_gtid_by_partial_command(thd);
     if (ret == 1)
@@ -2036,6 +2062,7 @@ bool mysql_alter_user(THD *thd, List <LEX_USER> &list, bool if_exists)
                               thd->rewritten_query.c_ptr_safe(),
                               thd->rewritten_query.length(),
                               table->file->has_transactions()) != 0);
+
     else if (ret == -1)
       result|= -1;
   }
