@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of the License.
@@ -185,9 +185,425 @@ TABLE_FIELD_TYPE mysql_db_table_fields[MYSQL_DB_FIELD_COUNT] = {
   }
 };
 
+static const
+TABLE_FIELD_TYPE mysql_user_table_fields[MYSQL_USER_FIELD_COUNT] = {
+  {
+    { C_STRING_WITH_LEN("Host") },
+    { C_STRING_WITH_LEN("char(60)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("User") },
+    { C_STRING_WITH_LEN("char(" USERNAME_CHAR_LENGTH_STR ")") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Select_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Insert_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Update_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Delete_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Create_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Drop_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Reload_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Shutdown_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Process_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("File_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Grant_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("References_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Index_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Alter_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Show_db_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Super_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Create_tmp_table_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Lock_tables_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Execute_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Repl_slave_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Repl_client_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Create_view_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Show_view_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Create_routine_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Alter_routine_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Create_user_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Event_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Trigger_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Create_tablespace_priv") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("ssl_type") },
+    { C_STRING_WITH_LEN("enum('','ANY','X509','SPECIFIED')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("ssl_cipher") },
+    { C_STRING_WITH_LEN("blob") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("x509_issuer") },
+    { C_STRING_WITH_LEN("blob") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("x509_subject") },
+    { C_STRING_WITH_LEN("blob") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("max_questions") },
+    { C_STRING_WITH_LEN("int(11)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("max_updates") },
+    { C_STRING_WITH_LEN("int(11)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("max_connections") },
+    { C_STRING_WITH_LEN("int(11)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("max_user_connections") },
+    { C_STRING_WITH_LEN("int(11)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("plugin") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("authentication_string") },
+    { C_STRING_WITH_LEN("text") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("password_expired") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("password_last_changed") },
+    { C_STRING_WITH_LEN("timestamp") },
+    { NULL, 0 }
+  },
+  {
+    { C_STRING_WITH_LEN("password_lifetime") },
+    { C_STRING_WITH_LEN("smallint(5)") },
+    { NULL, 0 }
+  },
+  {
+    { C_STRING_WITH_LEN("account_locked") },
+    { C_STRING_WITH_LEN("enum('N','Y')") },
+    { C_STRING_WITH_LEN("utf8") }
+  }
+};
+
+static const
+TABLE_FIELD_TYPE mysql_proxies_priv_table_fields[MYSQL_PROXIES_PRIV_FIELD_COUNT] = {
+  {
+   { C_STRING_WITH_LEN("Host") },
+    { C_STRING_WITH_LEN("char(60)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("User") },
+    { C_STRING_WITH_LEN("char(" USERNAME_CHAR_LENGTH_STR ")") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Proxied_host") },
+    { C_STRING_WITH_LEN("char(60)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Proxied_user") },
+    { C_STRING_WITH_LEN("char(" USERNAME_CHAR_LENGTH_STR ")") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("With_grant") },
+    { C_STRING_WITH_LEN("tinyint(1)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Grantor") },
+    { C_STRING_WITH_LEN("char(93)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Timestamp") },
+    { C_STRING_WITH_LEN("timestamp") },
+    {NULL, 0}
+  }
+};
+
+static const
+TABLE_FIELD_TYPE mysql_procs_priv_table_fields[MYSQL_PROCS_PRIV_FIELD_COUNT] = {
+  {
+    { C_STRING_WITH_LEN("Host") },
+    { C_STRING_WITH_LEN("char(60)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Db") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("User") },
+    { C_STRING_WITH_LEN("char(" USERNAME_CHAR_LENGTH_STR ")") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Routine_name") },
+    { C_STRING_WITH_LEN("char(64)") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Routine_type") },
+    { C_STRING_WITH_LEN("enum('FUNCTION','PROCEDURE')") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Grantor") },
+    { C_STRING_WITH_LEN("char(93)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Proc_priv") },
+    { C_STRING_WITH_LEN("set('Execute','Alter Routine','Grant')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Timestamp") },
+    { C_STRING_WITH_LEN("timestamp") },
+    {NULL, 0}
+  }
+};
+
+static const
+TABLE_FIELD_TYPE mysql_columns_priv_table_fields[MYSQL_COLUMNS_PRIV_FIELD_COUNT] = {
+  {
+    { C_STRING_WITH_LEN("Host") },
+    { C_STRING_WITH_LEN("char(60)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Db") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("User") },
+    { C_STRING_WITH_LEN("char(" USERNAME_CHAR_LENGTH_STR ")") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Table_name") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Column_name") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Timestamp") },
+    { C_STRING_WITH_LEN("timestamp") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Column_priv") },
+    { C_STRING_WITH_LEN("set('Select','Insert','Update','References')") },
+    { C_STRING_WITH_LEN("utf8") }
+  }
+};
+
+static const
+TABLE_FIELD_TYPE mysql_tables_priv_table_fields[MYSQL_TABLES_PRIV_FIELD_COUNT] = {
+  {
+    { C_STRING_WITH_LEN("Host") },
+    { C_STRING_WITH_LEN("char(60)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Db") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("User") },
+    { C_STRING_WITH_LEN("char(" USERNAME_CHAR_LENGTH_STR ")") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Table_name") },
+    { C_STRING_WITH_LEN("char(64)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Grantor") },
+    { C_STRING_WITH_LEN("char(93)") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Timestamp") },
+    { C_STRING_WITH_LEN("timestamp") },
+    {NULL, 0}
+  },
+  {
+    { C_STRING_WITH_LEN("Table_priv") },
+    { C_STRING_WITH_LEN("set('Select','Insert','Update','Delete','Create',"
+                        "'Drop','Grant','References','Index','Alter',"
+                        "'Create View','Show view','Trigger')") },
+    { C_STRING_WITH_LEN("utf8") }
+  },
+  {
+    { C_STRING_WITH_LEN("Column_priv") },
+    { C_STRING_WITH_LEN("set('Select','Insert','Update','References')") },
+    { C_STRING_WITH_LEN("utf8") }
+  }
+};
+
 const TABLE_FIELD_DEF
   mysql_db_table_def= {MYSQL_DB_FIELD_COUNT, mysql_db_table_fields};
 
+const TABLE_FIELD_DEF
+  mysql_user_table_def= {MYSQL_USER_FIELD_COUNT, mysql_user_table_fields};
+
+const TABLE_FIELD_DEF
+  mysql_proxies_priv_table_def= {MYSQL_PROXIES_PRIV_FIELD_COUNT,
+                                 mysql_proxies_priv_table_fields};
+
+const TABLE_FIELD_DEF
+  mysql_procs_priv_table_def= {MYSQL_PROCS_PRIV_FIELD_COUNT,
+                               mysql_procs_priv_table_fields};
+
+const TABLE_FIELD_DEF
+  mysql_columns_priv_table_def= {MYSQL_COLUMNS_PRIV_FIELD_COUNT,
+                                 mysql_columns_priv_table_fields};
+
+const TABLE_FIELD_DEF
+  mysql_tables_priv_table_def= {MYSQL_TABLES_PRIV_FIELD_COUNT,
+                                mysql_tables_priv_table_fields};
+
 
 #ifndef NO_EMBEDDED_ACCESS_CHECKS
 
@@ -461,6 +877,7 @@ void update_user_resource(TABLE *table, USER_RESOURCES *mqh)
       store((longlong) mqh->user_conn, TRUE);
 }
 
+
 int replace_user_table(THD *thd, TABLE *table, LEX_USER *combo,
                        ulong rights, bool revoke_grant,
                        bool can_create_user, ulong what_to_replace)
@@ -474,6 +891,7 @@ int replace_user_table(THD *thd, TABLE *table, LEX_USER *combo,
   LEX *lex= thd->lex;
   LEX_CSTRING old_plugin;
   struct timeval password_change_timestamp= {0, 0};
+  Acl_table_intact table_intact;
   DBUG_ENTER("replace_user_table");
 
   mysql_mutex_assert_owner(&acl_cache->lock);
@@ -484,13 +902,9 @@ int replace_user_table(THD *thd, TABLE *table, LEX_USER *combo,
       goto end;
   }
 
-  if (!table->key_info)
-  {
-    my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
-             table->s->table_name.str);
+  if (table_intact.check(table, &mysql_user_table_def))
     goto end;
-  }
-
+  
   table->use_all_columns();
   DBUG_ASSERT(combo->host.str != NULL);
   table->field[MYSQL_USER_FIELD_HOST]->store(combo->host.str,combo->host.length,
@@ -858,6 +1272,7 @@ int replace_db_table(TABLE *table, const char *db,
   int error;
   char what= (revoke_grant) ? 'N' : 'Y';
   uchar user_key[MAX_KEY_LENGTH];
+  Acl_table_intact table_intact;
   DBUG_ENTER("replace_db_table");
 
   if (!initialized)
@@ -866,6 +1281,9 @@ int replace_db_table(TABLE *table, const char *db,
     DBUG_RETURN(-1);
   }
 
+  if (table_intact.check(table, &mysql_db_table_def))
+    DBUG_RETURN(-1);
+
   /* Check if there is such a user in user table in memory? */
   if (!find_acl_user(combo.host.str,combo.user.str, FALSE))
   {
@@ -977,6 +1395,7 @@ int replace_proxies_priv_table(THD *thd, TABLE *table, const LEX_USER *user,
   uchar user_key[MAX_KEY_LENGTH];
   ACL_PROXY_USER new_grant;
   char grantor[USER_HOST_BUFF_SIZE];
+  Acl_table_intact table_intact;
 
   DBUG_ENTER("replace_proxies_priv_table");
 
@@ -986,6 +1405,9 @@ int replace_proxies_priv_table(THD *thd, TABLE *table, const LEX_USER *user,
     DBUG_RETURN(-1);
   }
 
+  if (table_intact.check(table, &mysql_proxies_priv_table_def))
+    DBUG_RETURN(-1);
+
   /* Check if there is such a user in user table in memory? */
   if (!find_acl_user(user->host.str,user->user.str, FALSE))
   {
@@ -1115,16 +1537,14 @@ int replace_column_table(GRANT_TABLE *g_t,
   int result=0;
   uchar key[MAX_KEY_LENGTH];
   uint key_prefix_length;
+  KEY_PART_INFO *key_part;
+  Acl_table_intact table_intact;
   DBUG_ENTER("replace_column_table");
-  
-  if (!table->key_info)
-  {
-    my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
-             table->s->table_name.str);
+
+  if (table_intact.check(table, &mysql_columns_priv_table_def))
     DBUG_RETURN(-1);
-  }
-  
-  KEY_PART_INFO *key_part= table->key_info->key_part;
+
+  key_part= table->key_info->key_part;
 
   table->use_all_columns();
   table->field[0]->store(combo.host.str,combo.host.length,
@@ -1372,8 +1792,12 @@ int replace_table_table(THD *thd, GRANT_TABLE *grant_table,
   int error=0;
   ulong store_table_rights, store_col_rights;
   uchar user_key[MAX_KEY_LENGTH];
+  Acl_table_intact table_intact;
   DBUG_ENTER("replace_table_table");
 
+  if (table_intact.check(table, &mysql_tables_priv_table_def))
+    DBUG_RETURN(-1);
+
   get_grantor(thd, grantor);
   /*
     The following should always succeed as new users are created before
@@ -1517,6 +1941,7 @@ int replace_routine_table(THD *thd, GRANT_NAME *grant_name,
   int old_row_exists= 1;
   int error=0;
   ulong store_proc_rights;
+  Acl_table_intact table_intact;
   DBUG_ENTER("replace_routine_table");
 
   if (!initialized)
@@ -1525,6 +1950,9 @@ int replace_routine_table(THD *thd, GRANT_NAME *grant_name,
     DBUG_RETURN(-1);
   }
 
+  if (table_intact.check(table, &mysql_procs_priv_table_def))
+    DBUG_RETURN(-1);
+
   get_grantor(thd, grantor);
 
   if (strlen(grantor) > (table->field[5]->field_length /
@@ -1731,6 +2159,25 @@ int open_grant_tables(THD *thd, TABLE_LIST *tables, bool *transactional_tables)
   }
 #endif
 
+#ifdef WITH_WSREP
+  /* Do NOT call for DROP PROCEDURE or DROP FUNCTION, these calls
+     are responsible for calling WSREP_TO_ISOLATION_BEGIN() themselves.
+   */
+  if (thd->lex->sql_command != SQLCOM_DROP_PROCEDURE &&
+      thd->lex->sql_command != SQLCOM_DROP_FUNCTION)
+  {
+    /*
+       Perform the TOI after the replication filter check to avoid
+       replicating commands that won't be applied locally (due to a filter).
+       WSREP_TO_ISOLATION_BEGIN
+    */
+    if (WSREP(thd) && wsrep_to_isolation_begin(thd, WSREP_MYSQL_DB, NULL, NULL))
+    {
+        DBUG_RETURN(-1);
+    }
+  }
+#endif /* WITH_WSREP */
+
   if (open_and_lock_tables(thd, tables, MYSQL_LOCK_IGNORE_TIMEOUT))
   {                                             // This should never happen
     DBUG_RETURN(-1);
@@ -1871,13 +2318,6 @@ int handle_grant_table(TABLE_LIST *tables, uint table_no, bool drop,
                       user_from->user.length,
                       system_charset_info);
     
-    if (!table->key_info)
-    {
-      my_error(ER_TABLE_CORRUPT, MYF(0), table->s->db.str,
-               table->s->table_name.str);
-      DBUG_RETURN(-1);
-    }
-
     key_prefix_length= (table->key_info->key_part[0].store_length +
                         table->key_info->key_part[1].store_length);
     key_copy(user_key, table->record[0], table->key_info, key_prefix_length);
