@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2005, 2015, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #include "lock.h"   // lock_object_name
 #include "log.h"
 #include "mysql/psi/mysql_sp.h"
+#include "debug_sync.h"
 
 /**
   @addtogroup Event_Scheduler
@@ -241,6 +242,8 @@ common_1_lev_code:
     break;
   case INTERVAL_WEEK:
     expr/= 7;
+    close_quote= FALSE;
+    break;
   default:
     close_quote= FALSE;
     break;
@@ -338,6 +341,8 @@ Events::create_event(THD *thd, Event_parse_data *parse_data,
     DBUG_RETURN(TRUE);
   }
 
+  WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, NULL)
+
   if (parse_data->do_not_create)
     DBUG_RETURN(FALSE);
   /* 
@@ -408,6 +413,11 @@ Events::create_event(THD *thd, Event_parse_data *parse_data,
   thd->variables.binlog_format= save_binlog_format;
 
   DBUG_RETURN(ret);
+
+#ifdef WITH_WSREP
+error:
+  DBUG_RETURN(TRUE);
+#endif /* WITH_WSREP */
 }
 
 
@@ -496,6 +506,8 @@ Events::update_event(THD *thd, Event_parse_data *parse_data,
     }
   }
 
+  WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, NULL)
+
   /* 
     Turn off row binlogging of this statement and use statement-based 
     so that all supporting tables are updated for UPDATE EVENT command.
@@ -511,6 +523,8 @@ Events::update_event(THD *thd, Event_parse_data *parse_data,
     LEX_STRING dbname= new_dbname ? *new_dbname : parse_data->dbname;
     LEX_STRING name= new_name ? *new_name : parse_data->name;
 
+    DEBUG_SYNC(thd, "after_alter_event_updated_event_table");
+
     if (opt_event_scheduler != Events::EVENTS_DISABLED)
     {
       if (!(new_element= new Event_queue_element()))
@@ -529,16 +543,17 @@ Events::update_event(THD *thd, Event_parse_data *parse_data,
         if (event_queue)
           event_queue->update_event(thd, parse_data->dbname, parse_data->name,
                                     new_element);
-        /* Binlog the alter event. */
-        DBUG_ASSERT(thd->query().str && thd->query().length);
-
-        thd->add_to_binlog_accessed_dbs(parse_data->dbname.str);
-        if (new_dbname)
-          thd->add_to_binlog_accessed_dbs(new_dbname->str);
-
-        ret= write_bin_log(thd, true, thd->query().str, thd->query().length);
       }
     }
+
+    /* Binlog the alter event. */
+    DBUG_ASSERT(thd->query().str && thd->query().length);
+
+    thd->add_to_binlog_accessed_dbs(parse_data->dbname.str);
+    if (new_dbname)
+      thd->add_to_binlog_accessed_dbs(new_dbname->str);
+
+    ret|= write_bin_log(thd, true, thd->query().str, thd->query().length);
   }
   /* Restore the state of binlog format */
   DBUG_ASSERT(!thd->is_current_stmt_binlog_format_row());
@@ -547,6 +562,11 @@ Events::update_event(THD *thd, Event_parse_data *parse_data,
   thd->variables.binlog_format= save_binlog_format;
 
   DBUG_RETURN(ret);
+
+#ifdef WITH_WSREP
+error:
+  DBUG_RETURN(TRUE);
+#endif /* WITH_WSREP */
 }
 
 
@@ -585,6 +605,7 @@ Events::drop_event(THD *thd, LEX_STRING dbname, LEX_STRING name, bool if_exists)
 
   if (check_access(thd, EVENT_ACL, dbname.str, NULL, NULL, 0, 0))
     DBUG_RETURN(TRUE);
+  WSREP_TO_ISOLATION_BEGIN(WSREP_MYSQL_DB, NULL, NULL)
 
   if (lock_object_name(thd, MDL_key::EVENT,
                        dbname.str, name.str))
@@ -606,6 +627,10 @@ Events::drop_event(THD *thd, LEX_STRING dbname, LEX_STRING name, bool if_exists)
 #endif 
   }
   DBUG_RETURN(ret);
+#ifdef WITH_WSREP
+ error:
+  DBUG_RETURN(TRUE);
+#endif /* WITH_WSREP */
 }
 
 
@@ -1273,6 +1298,7 @@ end:
 int wsrep_create_event_query(THD *thd, uchar** buf, size_t* buf_len)
 {
   String log_query;
+
   if (create_query_string(thd, &log_query))
   {
     WSREP_WARN("events create string failed: schema: %s, query: %s",
