@@ -1,7 +1,7 @@
 #ifndef LEX_INCLUDED
 #define LEX_INCLUDED
 
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -142,6 +142,7 @@ static const SYMBOL symbols[] = {
   { SYM("COMPLETION",               COMPLETION_SYM)},
   { SYM("COMPRESSION",              COMPRESSION_SYM)},
   { SYM("COMPRESSED",               COMPRESSED_SYM)},
+  { SYM("COMPRESSION_DICTIONARY",   COMPRESSION_DICTIONARY_SYM)},
   { SYM("ENCRYPTION",               ENCRYPTION_SYM)},
   { SYM("CONCURRENT",               CONCURRENT)},
   { SYM("CONDITION",                CONDITION_SYM)},
@@ -633,6 +634,7 @@ static const SYMBOL symbols[] = {
   { SYM("TOKUDB_SNAPPY",            TOKU_SNAPPY_SYM)},
   { SYM("TOKUDB_UNCOMPRESSED",      TOKU_UNCOMPRESSED_SYM)},
   { SYM("TOKUDB_ZLIB",              TOKU_ZLIB_SYM)},
+  { SYM("TOKUDB_DEFAULT",           TOKU_DEFAULT_SYM)},
   { SYM("TRAILING",                 TRAILING)},
   { SYM("TRANSACTION",              TRANSACTION_SYM)},
   { SYM("TRIGGER",                  TRIGGER_SYM)},
@@ -719,6 +721,8 @@ static const SYMBOL symbols[] = {
   { SYM_FN("DATE_SUB",              DATE_SUB_INTERVAL)},
   { SYM_FN("EXTRACT",               EXTRACT_SYM)},
   { SYM_FN("GROUP_CONCAT",          GROUP_CONCAT_SYM)},
+  { SYM_FN("JSON_OBJECTAGG",        JSON_OBJECTAGG)},
+  { SYM_FN("JSON_ARRAYAGG",         JSON_ARRAYAGG)},
   { SYM_FN("MAX",                   MAX_SYM)},
   { SYM_FN("MID",                   SUBSTRING)}, /* unireg function */
   { SYM_FN("MIN",                   MIN_SYM)},
