@@ -25,6 +25,26 @@
 #include "mysqld_thd_manager.h" // Global_THD_manager
 #include "sql_parse.h"          // Find_thd_with_id
 
+/**
+  Auxiliary function to stop all the running channel threads according to the
+  given mask.
+
+  @note: The caller shall possess channel_map lock before calling this function,
+         and unlock after returning from this function.
+
+  @param mi                   The pointer to Master_info instance
+  @param threads_to_stop      The types of threads to be stopped
+  @param timeout              The expected time in which the thread should stop
+
+  @return the operation status
+    @retval 0      OK
+    @retval !=0    Error
+*/
+int channel_stop(Master_info *mi,
+                 int threads_to_stop,
+                 long timeout);
+
+
 int initialize_channel_service_interface()
 {
   DBUG_ENTER("initialize_channel_service_interface");
@@ -163,10 +183,9 @@ initialize_channel_connection_info(Channel_connection_info* channel_info)
 void set_mi_ssl_options(LEX_MASTER_INFO* lex_mi, Channel_ssl_info* channel_ssl_info)
 {
 
-  if (channel_ssl_info->use_ssl)
-  {
-    lex_mi->ssl= LEX_MASTER_INFO::LEX_MI_ENABLE;
-  }
+  lex_mi->ssl= (channel_ssl_info->use_ssl) ?
+    LEX_MASTER_INFO::LEX_MI_ENABLE :
+    LEX_MASTER_INFO::LEX_MI_DISABLE;
 
   if (channel_ssl_info->ssl_ca_file_name != NULL)
   {
@@ -208,10 +227,9 @@ void set_mi_ssl_options(LEX_MASTER_INFO* lex_mi, Channel_ssl_info* channel_ssl_i
     lex_mi->ssl_cipher= channel_ssl_info->ssl_cipher;
   }
 
-  if (channel_ssl_info->ssl_verify_server_cert)
-  {
-    lex_mi->ssl_verify_server_cert= LEX_MASTER_INFO::LEX_MI_ENABLE;
-  }
+  lex_mi->ssl_verify_server_cert= (channel_ssl_info->ssl_verify_server_cert) ?
+    LEX_MASTER_INFO::LEX_MI_ENABLE :
+    LEX_MASTER_INFO::LEX_MI_DISABLE;
 }
 
 int channel_create(const char* channel,
@@ -246,8 +264,7 @@ int channel_create(const char* channel,
   /* create a new channel if doesn't exist */
   if (!mi)
   {
-    if ((error= add_new_channel(&mi, channel,
-                                channel_info->type)))
+    if ((error= add_new_channel(&mi, channel)))
         goto err;
   }
 
@@ -304,6 +321,14 @@ int channel_create(const char* channel,
 
   set_mi_settings(mi, channel_info);
 
+  if (channel_map.is_group_replication_channel_name(mi->get_channel()))
+  {
+    thd->variables.max_allowed_packet= slave_max_allowed_packet;
+    thd->get_protocol_classic()->set_max_packet_size(
+        slave_max_allowed_packet + MAX_LOG_EVENT_HEADER);
+  }
+
+
 err:
   channel_map.unlock();
 
@@ -436,19 +461,16 @@ err:
   DBUG_RETURN(error);
 }
 
-int channel_stop(const char* channel,
+int channel_stop(Master_info *mi,
                  int threads_to_stop,
                  long timeout)
 {
-  DBUG_ENTER("channel_stop(channel, stop_receiver, stop_applier, timeout");
+  DBUG_ENTER("channel_stop(master_info, stop_receiver, stop_applier, timeout");
 
-  channel_map.rdlock();
-
-  Master_info *mi= channel_map.get_mi(channel);
+  channel_map.assert_some_lock();
 
   if (mi == NULL)
   {
-    channel_map.unlock();
     DBUG_RETURN(RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR);
   }
 
@@ -485,7 +507,6 @@ int channel_stop(const char* channel,
 end:
   unlock_slave_threads(mi);
   mi->channel_unlock();
-  channel_map.unlock();
 
   if (thd_init)
   {
@@ -495,6 +516,64 @@ end:
   DBUG_RETURN(error);
 }
 
+int channel_stop(const char* channel,
+                 int threads_to_stop,
+                 long timeout)
+{
+  DBUG_ENTER("channel_stop(channel, stop_receiver, stop_applier, timeout");
+
+  channel_map.rdlock();
+
+  Master_info *mi= channel_map.get_mi(channel);
+
+  int error= channel_stop(mi, threads_to_stop, timeout);
+
+  channel_map.unlock();
+
+  DBUG_RETURN(error);
+}
+
+int channel_stop_all(int threads_to_stop,
+                     long timeout)
+{
+  DBUG_ENTER("channel_stop_all");
+
+  int error= 0;
+  Master_info *mi= 0;
+
+  channel_map.rdlock();
+
+  for (mi_map::iterator it= channel_map.begin(); it != channel_map.end(); it++)
+  {
+    mi= it->second;
+
+    if (mi)
+    {
+      DBUG_PRINT("info", ("stopping channel_name: %s",
+                          mi->get_channel()));
+
+      int channel_error= channel_stop(mi, threads_to_stop, timeout);
+
+      DBUG_EXECUTE_IF("group_replication_stop_all_channels_failure",
+                      {
+                        channel_error=1;
+                      });
+
+      if (channel_error &&
+          channel_error != RPL_CHANNEL_SERVICE_CHANNEL_DOES_NOT_EXISTS_ERROR)
+      {
+        error= channel_error;
+        mi->report(ERROR_LEVEL, error,
+                   "Error stopping channel: %s. Got error: %d",
+                   mi->get_channel(), error);
+      }
+    }
+  }
+
+  channel_map.unlock();
+  DBUG_RETURN(error);
+}
+
 int channel_purge_queue(const char* channel, bool reset_all)
 {
   DBUG_ENTER("channel_purge_queue(channel, only_purge");
@@ -971,4 +1050,14 @@ bool is_partial_transaction_on_channel_relay_log(const char *channel)
   channel_map.unlock();
   DBUG_RETURN(ret);
 }
+
+bool is_any_slave_channel_running(int thread_mask)
+{
+  bool status;
+  DBUG_ENTER("is_any_slave_channel_running");
+  channel_map.rdlock();
+  status= is_any_slave_channel_running(thread_mask, NULL);
+  channel_map.unlock();
+  DBUG_RETURN(status);
+}
 #endif /* HAVE_REPLICATION */
