@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -318,6 +318,7 @@ bool com_binlog_dump(THD *thd, char *packet, size_t packet_length)
   const uchar* packet_position= (uchar *) packet;
   size_t packet_bytes_todo= packet_length;
 
+  DBUG_ASSERT(!thd->status_var_aggregated);
   thd->status_var.com_other++;
   thd->enable_slow_log= opt_log_slow_admin_statements;
   if (check_global_access(thd, REPL_SLAVE_ACL))
@@ -368,6 +369,7 @@ bool com_binlog_dump_gtid(THD *thd, char *packet, size_t packet_length)
   Sid_map sid_map(NULL/*no sid_lock because this is a completely local object*/);
   Gtid_set slave_gtid_executed(&sid_map);
 
+  DBUG_ASSERT(!thd->status_var_aggregated);
   thd->status_var.com_other++;
   thd->enable_slow_log= opt_log_slow_admin_statements;
   if (check_global_access(thd, REPL_SLAVE_ACL))
@@ -565,6 +567,15 @@ bool reset_master(THD* thd)
 {
   bool ret= false;
 
+  /*
+    RESET MASTER command should ignore 'read-only' and 'super_read_only'
+    options so that it can update 'mysql.gtid_executed' replication repository
+    table.
+
+    Please note that skip_readonly_check flag should be set even when binary log
+    is not enabled, as RESET MASTER command will clear 'gtid_executed' table.
+  */
+  thd->set_skip_readonly_check();
   if (is_group_replication_running())
   {
     my_error(ER_CANT_RESET_MASTER, MYF(0), "Group Replication is running");
@@ -727,7 +738,8 @@ bool show_binlogs(THD* thd)
   
   cur_dir_len= dirname_length(cur.log_file_name);
 
-  reinit_io_cache(index_file, READ_CACHE, (my_off_t) 0, 0, 0);
+  if (reinit_io_cache(index_file, READ_CACHE, (my_off_t)0, 0, 0))
+    goto err;
 
   /* The file ends with EOF or empty line */
   while ((length=my_b_gets(index_file, fname, sizeof(fname))) > 1)
