@@ -22,6 +22,10 @@
 #include "mysqld_thd_manager.h"          // Global_THD_manager
 #include "sql_class.h"
 
+#ifdef WITH_WSREP
+#include "wsrep_mysqld.h"
+#endif
+
 #ifdef _WIN32
 #include <crtdbg.h>
 #define SIGNAL_FMT "exception 0x%x"
@@ -63,6 +67,14 @@ extern "C" void handle_fatal_signal(int sig)
 
   segfaulted = 1;
 
+/*
+  The wsrep subsystem has their its own actions
+  which need be performed before exiting:
+*/
+#ifdef WITH_WSREP
+  wsrep_handle_fatal_signal(sig);
+#endif
+
 #ifdef _WIN32
   SYSTEMTIME utc_time;
   GetSystemTime(&utc_time);
@@ -100,7 +112,7 @@ extern "C" void handle_fatal_signal(int sig)
     "As this is a crash and something is definitely wrong, the information\n"
     "collection process might fail.\n"
     "Please help us make Percona XtraDB Cluster better by reporting any\n"
-    "bugs at https://bugs.launchpad.net/percona-xtradb-cluster\n\n");
+    "bugs at https://jira.percona.com/projects/PXC/issues\n\n");
 
   my_safe_printf_stderr("key_buffer_size=%lu\n",
                         (ulong) dflt_key_cache->key_cache_mem_size);
