@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2002, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2002, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -790,6 +790,8 @@ bool sp_head::execute(THD *thd, bool merge_da_on_success)
 #ifdef WITH_WSREP
     if (thd->wsrep_next_trx_id() == WSREP_UNDEFINED_TRX_ID)
     {
+      if (thd->query_id == 0)
+        thd->set_query_id(next_query_id());
       thd->set_wsrep_next_trx_id(thd->query_id);
       WSREP_DEBUG("Assigned new next trx-id (%lu) to Store-Procedure execution",
                   (long unsigned int) thd->wsrep_next_trx_id());
@@ -860,7 +862,15 @@ bool sp_head::execute(THD *thd, bool merge_da_on_success)
   DBUG_ASSERT(thd->change_list.is_empty());
   old_change_list.move_elements_to(&thd->change_list);
   thd->lex= old_lex;
-  thd->set_query_id(old_query_id);
+
+#ifdef WITH_WSREP
+  /* Avoid updating wsrep_next_trx_id on completion of Store-Proc.
+     It should be updated only before the real action. */
+  thd->set_query_id(old_query_id, false);
+#else
+   thd->set_query_id(old_query_id);
+#endif /* WITH_WSREP */
+
   DBUG_ASSERT(!thd->derived_tables);
   thd->derived_tables= old_derived_tables;
   thd->variables.sql_mode= save_sql_mode;
@@ -1982,8 +1992,7 @@ bool sp_head::show_routine_code(THD *thd)
     if (ip != i->get_ip())
     {
       const char *format= "Instruction at position %u has m_ip=%u";
-      char tmp[sizeof(format) + 2 * sizeof(uint) + 1];
-
+      char tmp[64 + 2 * MY_INT32_NUM_DECIMAL_DIGITS];
       sprintf(tmp, format, ip, i->get_ip());
       /*
         Since this is for debugging purposes only, we don't bother to
@@ -2182,7 +2191,7 @@ void sp_head::add_used_tables_to_table_list(THD *thd,
 bool sp_head::check_show_access(THD *thd, bool *full_access)
 {
   TABLE_LIST tables;
-  memset(&tables, 0, sizeof(tables));
+  memset(static_cast<void*>(&tables), 0, sizeof(tables));
   tables.db= (char*) "mysql";
   tables.table_name= tables.alias= (char*) "proc";
 
