@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2015, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2010, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -14,6 +14,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
 #include "sql_alter.h"
+#ifdef WITH_WSREP
+#include "wsrep_mysqld.h"
+#include "sql_parse.h"
+#endif /* WITH_WSREP */
 
 #include "auth_common.h"                     // check_access
 #include "sql_table.h"                       // mysql_alter_table,
@@ -25,6 +29,7 @@
 #endif /* WITH_WSREP */
 #include "log.h"
 
+bool has_external_data_or_index_dir(partition_info &pi);
 
 Alter_info::Alter_info(const Alter_info &rhs, MEM_ROOT *mem_root)
   :drop_list(rhs.drop_list, mem_root),
@@ -93,6 +98,24 @@ bool Alter_info::set_requested_lock(const LEX_STRING *str)
   return false;
 }
 
+/**
+  Checks if there are any columns with COLUMN_FORMAT COMRPESSED
+  attribute among field definitions in create_list.
+
+  @retval false there are no compressed columns
+  @retval true there is at least one compressed column
+*/
+bool Alter_info::has_compressed_columns() const
+{
+  List_iterator<Create_field> it(
+    const_cast<List<Create_field>&>(create_list));
+  const Create_field* sql_field;
+  while ((sql_field= it++))
+    if (sql_field->column_format() == COLUMN_FORMAT_TYPE_COMPRESSED)
+      return true;
+  return false;
+}
+
 
 Alter_table_ctx::Alter_table_ctx()
   : datetime_field(NULL), error_if_not_empty(false),
@@ -221,6 +244,14 @@ bool Sql_cmd_alter_table::execute(THD *thd)
 
   if (thd->is_fatal_error) /* out of memory creating a copy of alter_info */
     DBUG_RETURN(TRUE);
+
+  {
+    partition_info *part_info= thd->lex->part_info;
+    if (part_info != NULL && has_external_data_or_index_dir(*part_info) &&
+        check_access(thd, FILE_ACL, any_db, NULL, NULL, FALSE, FALSE))
+
+      DBUG_RETURN(TRUE);
+  }
   /*
     We also require DROP priv for ALTER TABLE ... DROP PARTITION, as well
     as for RENAME TO, as being done by SQLCOM_RENAME_TABLE
@@ -293,7 +324,7 @@ bool Sql_cmd_alter_table::execute(THD *thd)
   {
     // Rename of table
     TABLE_LIST tmp_table;
-    memset(&tmp_table, 0, sizeof(tmp_table));
+    memset(static_cast<void*>(&tmp_table), 0, sizeof(tmp_table));
     tmp_table.table_name= lex->name.str;
     tmp_table.db= select_lex->db;
     tmp_table.grant.privilege= priv;
@@ -313,7 +344,7 @@ bool Sql_cmd_alter_table::execute(THD *thd)
                         "INDEX DIRECTORY");
   create_info.data_file_name= create_info.index_file_name= NULL;
 
-  thd->enable_slow_log= opt_log_slow_admin_statements;
+  thd->set_slow_log_for_admin_command();
 
 
 #ifdef WITH_WSREP
@@ -511,6 +542,47 @@ bool Sql_cmd_discard_import_tablespace::execute(THD *thd)
   /* first table of first SELECT_LEX */
   TABLE_LIST *table_list= select_lex->get_table_list();
 
+  /* Disable DISCARD/IMPORT TABLESPACE as this command
+  will execute on single node and can introduce data
+  in-consistency in cluster. */
+  bool block= false;
+  switch(pxc_strict_mode)
+  {
+  case PXC_STRICT_MODE_DISABLED:
+    break;
+  case PXC_STRICT_MODE_PERMISSIVE:
+     WSREP_WARN("Percona-XtraDB-Cluster doesn't recommend"
+                " DISCARD/IMPORT of tablespace as it can introduce"
+                " data in-consistency (if not done in tandem on all nodes)"
+                " with pxc_strict_mode = PERMISSIVE");
+     push_warning_printf(
+        thd, Sql_condition::SL_WARNING, ER_UNKNOWN_ERROR,
+        "Percona-XtraDB-Cluster doesn't recommend"
+        " DISCARD/IMPORT of tablespace as it can introduce"
+        " data in-consistency (if not done in tandem on all nodes)"
+        " with pxc_strict_mode = PERMISSIVE");
+      break;
+  case PXC_STRICT_MODE_ENFORCING:
+  case PXC_STRICT_MODE_MASTER:
+  default:
+     block= true;
+     WSREP_ERROR("Percona-XtraDB-Cluster doesn't recommend"
+                 " DISCARD/IMPORT of tablespace as it can introduce"
+                 " data in-consistency (if not done in tandem on all nodes)"
+                 " with pxc_strict_mode = ENFORCING or MASTER");
+     char message[1024];
+     sprintf(message,
+             "Percona-XtraDB-Cluster prohibits"
+             " DISCARD/IMPORT of tablespace as it can introduce"
+             " data in-consistency (if not done in tandem on all nodes)"
+             " with pxc_strict_mode = ENFORCING or MASTER");
+     my_message(ER_UNKNOWN_ERROR, message, MYF(0));
+     break;
+  }
+
+  if (block)
+    return true;
+
   if (check_access(thd, ALTER_ACL, table_list->db,
                    &table_list->grant.privilege,
                    &table_list->grant.m_internal,
@@ -520,7 +592,7 @@ bool Sql_cmd_discard_import_tablespace::execute(THD *thd)
   if (check_grant(thd, ALTER_ACL, table_list, false, UINT_MAX, false))
     return true;
 
-  thd->enable_slow_log= opt_log_slow_admin_statements;
+  thd->set_slow_log_for_admin_command();
 
   /*
     Check if we attempt to alter mysql.slow_log or
