@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2007, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2007, 2018, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -448,9 +448,8 @@ static int increment_count_by_name(const char *name, const char *role_name,
       return 1; // Out of memory
     }
   }
-  user_stats->concurrent_connections++;
   user_stats->total_connections++;
-  if (thd->get_protocol()->get_ssl())
+  if (thd->get_ssl())
     user_stats->total_ssl_connections++;
   return 0;
 }
@@ -497,7 +496,7 @@ static int increment_count_by_id(my_thread_id id,
     }
   }
   thread_stats->total_connections++;
-  if (thd->get_protocol()->get_ssl())
+  if (thd->get_ssl())
     thread_stats->total_ssl_connections++;
   return 0;
 }
@@ -573,11 +572,6 @@ static void update_global_user_stats_with_user(THD* thd,
   user_stats->lost_connections+=     thd->diff_lost_connections;
   user_stats->access_denied_errors+= thd->diff_access_denied_errors;
   user_stats->empty_queries+=        thd->diff_empty_queries;
-
-  if (thd->diff_disconnects && thd->diff_denied_connections == 0) {
-    DBUG_ASSERT(user_stats->concurrent_connections > 0);
-    user_stats->concurrent_connections-=  thd->diff_disconnects;
-  }
 }
 
 static void update_global_thread_stats_with_thread(THD* thd,
@@ -686,6 +680,54 @@ void update_global_user_stats(THD* thd, bool create_user, time_t now)
   mysql_mutex_unlock(&LOCK_global_user_client_stats);
 }
 
+static void clear_stats_concurrent_connections(HASH* stats)
+{
+  for (ulong idx= 0; idx < stats->records; idx++)
+  {
+    USER_STATS* const user_stats=
+      reinterpret_cast<USER_STATS*>(my_hash_element(stats, idx));
+    user_stats->concurrent_connections= 0;
+  }
+}
+
+static void inc_stats_concurrent_conn(HASH* stats,
+				      const char* user_string, int cnt)
+{
+  USER_STATS* const user_stats=
+    reinterpret_cast<USER_STATS*>(my_hash_search(stats,
+          reinterpret_cast<const uchar*>(user_string),
+          strlen(user_string)));
+  if (user_stats)
+    user_stats->concurrent_connections+= cnt;
+}
+
+/**
+  Update number of concurrent connections for user_stats and client_stats
+  based on account resource limits
+*/
+void refresh_concurrent_conn_stats()
+{
+  mysql_mutex_lock(&LOCK_user_conn);
+
+  mysql_mutex_lock(&LOCK_global_user_client_stats);
+  clear_stats_concurrent_connections(&global_user_stats);
+  clear_stats_concurrent_connections(&global_client_stats);
+  mysql_mutex_unlock(&LOCK_global_user_client_stats);
+
+  for (ulong idx= 0; idx < hash_user_connections.records; idx++)
+  {
+    const struct user_conn* const uc=
+      reinterpret_cast<struct user_conn*>(
+          my_hash_element(&hash_user_connections, idx));
+    mysql_mutex_lock(&LOCK_global_user_client_stats);
+    inc_stats_concurrent_conn(&global_user_stats, uc->user, uc->connections);
+    inc_stats_concurrent_conn(&global_client_stats, uc->host, uc->connections);
+    mysql_mutex_unlock(&LOCK_global_user_client_stats);
+  }
+
+  mysql_mutex_unlock(&LOCK_user_conn);
+}
+
 /*
   check if user has already too many connections
 
@@ -900,6 +942,10 @@ void release_user_connection(THD *thd)
   DBUG_VOID_RETURN;
 }
 
+void refresh_concurrent_conn_stats()
+{
+}
+
 #endif /* NO_EMBEDDED_ACCESS_CHECKS */
 
 /*
@@ -1266,6 +1312,14 @@ static int check_connection(THD *thd, bool extra_port_connection)
     reset_host_connect_errors(thd->m_main_security_ctx.ip().str);
   }
 
+  /*
+    Now that acl_authenticate() is executed,
+    the SSL info is available.
+    Advertise it to THD, so SSL status variables
+    can be inspected.
+  */
+  thd->set_ssl(net->vio);
+
   return auth_rc;
 }
 
@@ -1364,12 +1418,6 @@ void end_connection(THD *thd)
   */
   release_user_connection(thd);
 
-  if (unlikely(opt_userstat)) {
-    thd->update_stats(false);
-    thd->diff_disconnects= 1;
-    update_global_user_stats(thd, false, time(NULL));
-  }
-
   if (thd->killed || (net->error && net->vio != 0))
   {
     aborted_threads++;
@@ -1422,7 +1470,16 @@ static void prepare_new_connection_state(THD* thd)
 
   if (opt_init_connect.length && !(sctx->check_access(SUPER_ACL)))
   {
+    if (sctx->password_expired())
+    {
+      sql_print_warning("init_connect variable is ignored for user: %s "
+                        "host: %s due to expired password.", sctx->priv_user().str,
+                        sctx->priv_host().str);
+      return;
+    }
+
     execute_init_command(thd, &opt_init_connect, &LOCK_sys_init_connect);
+
     if (thd->is_error())
     {
       Host_errors errors;
