@@ -167,20 +167,23 @@ static int wsrep_write_cache_once(wsrep_t*  const wsrep,
 
     uint length(my_b_bytes_in_cache(cache));
 
-    /* Start 5.7 MySQL server will write GTID directly to binlog file.
-    This means GTID event is not present in the case if GTID is set explictly.
-    We have cached the said GTID event which is then written before filling up
-    the cache with the real-data-change event. */
+    /* If galera node is acting as independent slave then GTID event that is
+    captured during processing of relay log should be cached and appended to
+    replicating write-set to ensure all the nodes of cluster are using
+    GTID sequence. */
     if (thd->wsrep_gtid_event_buf)
     {
-      if (thd->wsrep_gtid_event_buf_len < allocated)
+      if (thd->wsrep_gtid_event_buf_len < (allocated - used))
       {
-        memcpy(stack_buf, thd->wsrep_gtid_event_buf, thd->wsrep_gtid_event_buf_len);
+        memcpy(buf + used,
+               thd->wsrep_gtid_event_buf,
+               thd->wsrep_gtid_event_buf_len);
       }
       else
       {
+        size_t const new_size(heap_size(thd->wsrep_gtid_event_buf_len + used));
         uchar* tmp = (uchar *)my_realloc(key_memory_wsrep, heap_buf,
-                                         thd->wsrep_gtid_event_buf_len, MYF(0));
+                                         new_size, MYF(0));
         if (!tmp)
         {
           WSREP_ERROR("Failed to allocate/reallocate buffer to hold GTID event"
@@ -189,15 +192,37 @@ static int wsrep_write_cache_once(wsrep_t*  const wsrep,
           err = WSREP_TRX_SIZE_EXCEEDED;
           goto cleanup;
         }
-        heap_buf = tmp;
-        buf = heap_buf;
-        allocated = thd->wsrep_gtid_event_buf_len;
 
-        memcpy(buf, thd->wsrep_gtid_event_buf, thd->wsrep_gtid_event_buf_len);
+        /* Copy over existing buffer content. */
+        heap_buf = tmp;
+        memcpy(heap_buf, buf, used);
+        buf = heap_buf;
+        allocated = new_size;
+
+        memcpy(buf + used,
+               thd->wsrep_gtid_event_buf,
+               thd->wsrep_gtid_event_buf_len);
       }
+
       total_length += thd->wsrep_gtid_event_buf_len;
       used = total_length;
     }
+    else if (thd->variables.gtid_next.type != AUTOMATIC_GROUP)
+    {
+      /* Starting 5.7, MySQL delays appending GTID to binlog.
+      It is done at commit time. pre-commit hook doesn't have the GTID
+      information. If user has set explict GTID using gtid_next=UUID:seqno
+      then such event should be appended to write-set. */
+      Gtid_log_event gtid_event(thd, true, 0, 0, false);
+      uchar gtid_buf[Gtid_log_event::MAX_EVENT_LENGTH];
+      uint32 gtid_len = gtid_event.write_to_memory(gtid_buf);
+
+      assert(Gtid_log_event::MAX_EVENT_LENGTH < STACK_SIZE);
+
+      memcpy(buf + used, gtid_buf, gtid_len);
+      used += gtid_len;
+      total_length += gtid_len;
+    }
 
     if (unlikely(0 == length)) length = my_b_fill(cache);
 
@@ -237,10 +262,7 @@ static int wsrep_write_cache_once(wsrep_t*  const wsrep,
             allocated = new_size;
 
             if (used <= STACK_SIZE && used > 0) // there's data in stack_buf
-            {
-                DBUG_ASSERT(buf == stack_buf);
                 memcpy(heap_buf, stack_buf, used);
-            }
         }
 
         memcpy(buf + used, cache->read_pos, length);
@@ -292,17 +314,33 @@ static int wsrep_write_cache_inc(wsrep_t*  const wsrep,
     int err(WSREP_OK);
 
     size_t total_length(*len);
-
     uint length(my_b_bytes_in_cache(cache));
+
     if (thd->wsrep_gtid_event_buf)
     {
-        if(WSREP_OK != (err=wsrep_append_data(wsrep, &thd->wsrep_ws_handle,
-                                              thd->wsrep_gtid_event_buf,
-                                              thd->wsrep_gtid_event_buf_len)))
-                goto cleanup;
-
-        total_length += thd->wsrep_gtid_event_buf_len;
+      if (WSREP_OK != (err=wsrep_append_data(wsrep, &thd->wsrep_ws_handle,
+                                             thd->wsrep_gtid_event_buf,
+                                             thd->wsrep_gtid_event_buf_len)))
+        goto cleanup;
+      total_length += thd->wsrep_gtid_event_buf_len;
     }
+    else if (total_length == 0 && thd->variables.gtid_next.type != AUTOMATIC_GROUP)
+    {
+      /* Starting 5.7, MySQL delays appending GTID to binlog. It is done at
+      commit time. pre-commit hook doesn't have the GTID information.
+      If user has set explict GTID using gtid_next=UUID:seqno then such event
+      should be appended to write-set. */
+      Gtid_log_event gtid_event(thd, true, 0, 0, false);
+      uchar gtid_buf[Gtid_log_event::MAX_EVENT_LENGTH];
+      uint32 gtid_len = gtid_event.write_to_memory(gtid_buf);
+
+      if (WSREP_OK != (err=wsrep_append_data(wsrep, &thd->wsrep_ws_handle,
+                                            gtid_buf, gtid_len)))
+        goto cleanup;
+
+      total_length += gtid_len;
+    }
+
     if (unlikely(0 == length)) length = my_b_fill(cache);
 
     if (likely(length > 0)) do
