@@ -13,6 +13,7 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */
 
+#include <sql_plugin.h> // SHOW_MY_BOOL
 #include <mysqld.h>
 #include <sql_base.h>
 #include <sql_class.h>
@@ -83,6 +84,10 @@ my_bool wsrep_restart_slave_activated  = 0; // node has dropped, and slave
                                             // restart will be needed
 my_bool wsrep_slave_UK_checks          = 0; // slave thread does UK checks
 my_bool wsrep_slave_FK_checks          = 0; // slave thread does FK checks
+ulong   wsrep_RSU_commit_timeout       = 5000; // wait for x micr-secs
+                                               // to allow active connection to
+                                               // commit before starting RSU.
+
 
 /* pxc-strict-mode help control behavior of experimental features like
 myisam table replication, etc... */
@@ -676,7 +681,7 @@ wsrep_view_handler_cb (void*                    app_ctx,
       // version change
       if (view->proto_ver != wsrep_protocol_version)
       {
-          my_bool wsrep_ready_saved= wsrep_ready;
+          my_bool wsrep_ready_saved= wsrep_ready_get();
           wsrep_ready_set(FALSE);
           WSREP_INFO("closing client connections for "
                      "protocol change %ld -> %d",
@@ -766,6 +771,13 @@ wsrep_view_handler_cb (void*                    app_ctx,
 
   if (wsrep_auto_increment_control)
   {
+    WSREP_INFO("Auto Increment Offset/Increment re-align with cluster"
+                " membership change (Offset: %lu -> %lu)"
+                " (Increment: %lu -> %lu)",
+                global_system_variables.auto_increment_offset,
+                (long unsigned) view->my_idx + 1,
+                global_system_variables.auto_increment_increment,
+                (long unsigned) view->memb_num);
     global_system_variables.auto_increment_offset= view->my_idx + 1;
     global_system_variables.auto_increment_increment= view->memb_num;
   }
@@ -789,16 +801,34 @@ out:
   return WSREP_CB_SUCCESS;
 }
 
-void wsrep_ready_set (my_bool x)
+my_bool wsrep_ready_set (my_bool x)
 {
   WSREP_INFO("Setting wsrep_ready to %s", (x ? "true" : "false"));
   if (mysql_mutex_lock (&LOCK_wsrep_ready)) abort();
-  if (wsrep_ready != x)
+  my_bool ret= (wsrep_ready != x);
+  if (ret)
   {
     wsrep_ready= x;
     mysql_cond_broadcast (&COND_wsrep_ready);
   }
   mysql_mutex_unlock (&LOCK_wsrep_ready);
+  return ret;
+}
+
+my_bool wsrep_ready_get (void)
+{
+  if (mysql_mutex_lock (&LOCK_wsrep_ready)) abort();
+  my_bool ret= wsrep_ready;
+  mysql_mutex_unlock (&LOCK_wsrep_ready);
+  return ret;
+}
+
+int wsrep_show_ready(THD *thd, SHOW_VAR *var, char *buff)
+{
+  var->type= SHOW_MY_BOOL;
+  var->value= buff;
+  *((my_bool *)buff)= wsrep_ready_get();
+  return 0;
 }
 
 // Wait until wsrep has reached ready state
@@ -817,18 +847,8 @@ void wsrep_ready_wait ()
 static void wsrep_synced_cb(void* app_ctx)
 {
   WSREP_INFO("Synchronized with group, ready for connections");
-  bool signal_main= false;
-  if (mysql_mutex_lock (&LOCK_wsrep_ready)) abort();
-  if (!wsrep_ready)
-  {
-    WSREP_INFO("This node is synced, setting wsrep_ready to true");
-    wsrep_ready= TRUE;
-    mysql_cond_broadcast (&COND_wsrep_ready);
-    signal_main= true;
-
-  }
+  my_bool signal_main= wsrep_ready_set(TRUE);
   local_status.set(WSREP_MEMBER_SYNCED);
-  mysql_mutex_unlock (&LOCK_wsrep_ready);
 
   if (signal_main)
   {
@@ -897,18 +917,29 @@ int wsrep_init()
   wsrep_ready_set(FALSE);
   assert(wsrep_provider);
 
+  // Reset global settings
+  wsrep_incremental_data_collection= FALSE;
+
   wsrep_init_position();
 
   if ((rcode= wsrep_load(wsrep_provider, &wsrep, wsrep_log_cb)) != WSREP_OK)
   {
-    if (strcasecmp(wsrep_provider, WSREP_NONE))
+    if (strlen(wsrep_provider) == 0)
     {
-      WSREP_ERROR("Failed to load wsrep_provider (%s). Error: %s (code: %d)."
-                  " Reverting to no provider.",
+      WSREP_ERROR("wsrep_load() failed to load the provider('%s'): %s (%d). Reverting to no provider.",
                   wsrep_provider, strerror(rcode), rcode);
-      strcpy((char*)wsrep_provider, WSREP_NONE); // damn it's a dirty hack
+
+      if (wsrep_provider) my_free((void *)wsrep_provider);
+      wsrep_provider = my_strdup(key_memory_wsrep, WSREP_NONE, MYF(0)); // damn it's a dirty hack
+
       return wsrep_init();
     }
+    else if (strcasecmp(wsrep_provider, WSREP_NONE))
+    {
+      WSREP_ERROR("wsrep_load() failed to load the provider('%s'): %s (%d). Need to abort.",
+                  wsrep_provider, strerror(rcode), rcode);
+      unireg_abort(1);
+    }
     else /* this is for recursive call above */
     {
       WSREP_ERROR("Could not revert to no provider: %s (%d). Need to abort.",
@@ -1239,7 +1270,7 @@ void wsrep_recover()
 }
 
 
-void wsrep_stop_replication(THD *thd)
+void wsrep_stop_replication(THD *thd, bool is_server_shutdown)
 {
   WSREP_INFO("Stop replication");
   if (!wsrep)
@@ -1254,7 +1285,12 @@ void wsrep_stop_replication(THD *thd)
 
   wsrep_connected= FALSE;
 
-  wsrep_close_client_connections(TRUE, false);
+  /*
+   * On shutdown, let the normal mysqld shutdown code close the client
+   * connections.  See signal_hand() in mysqld.cc.
+  */
+  if (!is_server_shutdown)
+    wsrep_close_client_connections(TRUE, false);
 
   /* wait until appliers have stopped */
   wsrep_wait_appliers_close(thd);
@@ -1860,7 +1896,7 @@ static bool wsrep_can_run_in_toi(THD *thd, const char *db, const char *table,
       return true;
     }
 
-    if (table_list)
+    if (table_list && first_table)
     {
       for (TABLE_LIST* table= first_table; table; table= table->next_global)
       {
@@ -1870,7 +1906,7 @@ static bool wsrep_can_run_in_toi(THD *thd, const char *db, const char *table,
         }
       }
     }
-    return !(table || table_list);
+    return !(table || (table_list && first_table));
   }
 }
 
@@ -1888,6 +1924,7 @@ static int wsrep_TOI_begin(THD *thd, const char *db_, const char *table_,
   size_t buf_len(0);
   int buf_err;
 
+  thd->wsrep_skip_wsrep_hton= true;
   if (wsrep_can_run_in_toi(thd, db_, table_, table_list) == false)
   {
     WSREP_DEBUG("Can't execute %s in TOI mode", WSREP_QUERY(thd));
@@ -1991,7 +2028,7 @@ static int wsrep_TOI_begin(THD *thd, const char *db_, const char *table_,
     thd->wsrep_gtid_event_buf_len = 0;
     thd->wsrep_gtid_event_buf     = NULL;
     wsrep_keys_free(&key_arr);
-
+    wsrep_cleanup_transaction(thd);
     return -1;
   }
   else {
@@ -2005,7 +2042,6 @@ static int wsrep_TOI_begin(THD *thd, const char *db_, const char *table_,
     thd->wsrep_gtid_event_buf_len = 0;
     thd->wsrep_gtid_event_buf     = NULL;
     wsrep_keys_free(&key_arr);
-
     return 1;
   }
 
@@ -2068,7 +2104,7 @@ static int wsrep_RSU_begin(THD *thd, const char *db_, const char *table_)
                " for schema: %s, query: %s",
                ret, (thd->db().str ? thd->db().str : "(null)"), WSREP_QUERY(thd));
     my_error(ER_LOCK_DEADLOCK, MYF(0));
-    return(ret);
+    return(-1);
   }
 
   mysql_mutex_lock(&LOCK_wsrep_replaying);
@@ -2093,7 +2129,7 @@ static int wsrep_RSU_begin(THD *thd, const char *db_, const char *table_)
     }
 
     my_error(ER_LOCK_DEADLOCK, MYF(0));
-    return(1);
+    return(-1);
   }
 
   seqno = wsrep->pause(wsrep);
@@ -2105,7 +2141,7 @@ static int wsrep_RSU_begin(THD *thd, const char *db_, const char *table_)
     /* Pause fail so rollback desync action too. */
     wsrep->resync(wsrep);
 
-    return(1);
+    return(-1);
   }
   thd->global_read_lock.pause_provider(true);
   WSREP_DEBUG("Provider paused for RSU processing at seqno: %lld",
@@ -2270,6 +2306,9 @@ void wsrep_to_isolation_end(THD *thd)
     }
     wsrep_cleanup_transaction(thd);
   }
+
+  if (thd->wsrep_skip_wsrep_hton)
+    thd->wsrep_skip_wsrep_hton= false;
 }
 
 #define WSREP_MDL_LOG(severity, msg, schema, schema_len, req, gra)             \
