@@ -109,6 +109,8 @@ extern my_bool     wsrep_restart_slave;
 extern my_bool     wsrep_restart_slave_activated;
 extern my_bool     wsrep_slave_FK_checks;
 extern my_bool     wsrep_slave_UK_checks;
+extern ulong       wsrep_running_threads;
+extern ulong       wsrep_RSU_commit_timeout;
 
 enum enum_wsrep_reject_types {
   WSREP_REJECT_NONE,    /* nothing rejected */
@@ -164,6 +166,7 @@ extern const char* wsrep_provider_version;
 extern const char* wsrep_provider_vendor;
 
 int  wsrep_show_status(THD *thd, SHOW_VAR *var, char *buff);
+int  wsrep_show_ready(THD *thd, SHOW_VAR *var, char *buff);
 void wsrep_free_status(THD *thd);
 
 /* Filters out --wsrep-new-cluster oprtion from argv[]
@@ -193,6 +196,7 @@ extern "C" void wsrep_thd_set_query_state(
         THD *thd, enum wsrep_query_state state);
 extern "C" void wsrep_thd_set_conflict_state(
         THD *thd, bool lock, enum wsrep_conflict_state state);
+extern "C" void wsrep_thd_mark_split_trx(THD* thd, bool mini_trx);
 
 extern "C" void wsrep_thd_set_trx_to_replay(THD *thd, uint64 trx_id);
 
@@ -230,7 +234,7 @@ extern void wsrep_close_applier_threads(int count);
 extern void wsrep_kill_mysql(THD *thd);
 
 /* new defines */
-extern void wsrep_stop_replication(THD *thd);
+extern void wsrep_stop_replication(THD *thd, bool is_server_shutdown);
 extern bool wsrep_start_replication();
 extern bool wsrep_must_sync_wait (THD* thd, uint mask = WSREP_SYNC_WAIT_BEFORE_READ);
 extern bool wsrep_sync_wait (THD* thd, uint mask = WSREP_SYNC_WAIT_BEFORE_READ);
@@ -304,6 +308,7 @@ extern wsrep_seqno_t wsrep_locked_seqno;
   ((!opt_general_log_raw) && thd->rewritten_query.length()      \
    ? thd->rewritten_query.c_ptr_safe() : thd->query().str)
 
+extern my_bool wsrep_ready_get();
 extern void wsrep_ready_wait();
 
 enum wsrep_trx_status {
