@@ -88,6 +88,18 @@ bool wsrep_sst_method_check (sys_var *self, THD* thd, set_var* var)
     return true;
   }
 
+  if (!strcmp (var->save_result.string_value.str, WSREP_SST_MYSQLDUMP))
+  {
+    WSREP_WARN("Percona-XtraDB-Cluster has deprecated SST through mysqldump."
+               " Percona-XtraDB-Cluster recommends using xtrabackup."
+               " Please switch to use xtrabackup or rsync.");
+    push_warning_printf(
+      thd, Sql_condition::SL_WARNING, ER_UNKNOWN_ERROR,
+      "Percona-XtraDB-Cluster has deprecated SST through mysqldump."
+      " Percona-XtraDB-Cluster recommends using xtrabackup."
+      " Please switch to use xtrabackup or rsync.");
+  }
+
   return false;
 }
 
@@ -212,8 +224,6 @@ bool wsrep_sst_donor_update (sys_var *self, THD* thd, enum_var_type type)
     return 0;
 }
 
-static wsrep_uuid_t cluster_uuid = WSREP_UUID_UNDEFINED;
-
 bool wsrep_before_SE()
 {
   return (wsrep_provider != NULL
@@ -787,18 +797,19 @@ ssize_t wsrep_sst_prepare (void** msg, THD* thd)
 
   // Figure out SST address. Common for all SST methods
   if (wsrep_sst_receive_address &&
-    strcmp (wsrep_sst_receive_address, WSREP_SST_ADDRESS_AUTO))
+      strcmp (wsrep_sst_receive_address, WSREP_SST_ADDRESS_AUTO))
   {
     addr_in= wsrep_sst_receive_address;
   }
   else if (wsrep_node_address && strlen(wsrep_node_address))
   {
-    const char* const colon= strchr (wsrep_node_address, ':');
-    if (colon)
+    size_t const addr_len= strlen(wsrep_node_address);
+    size_t const host_len= wsrep_host_len(wsrep_node_address, addr_len);
+
+    if (host_len < addr_len)
     {
-      ptrdiff_t const len= colon - wsrep_node_address;
-      strncpy (ip_buf, wsrep_node_address, len);
-      ip_buf[len]= '\0';
+      strncpy (ip_buf, wsrep_node_address, host_len);
+      ip_buf[host_len]= '\0';
       addr_in= ip_buf;
     }
     else
@@ -951,25 +962,6 @@ static int sst_donate_mysqldump (const char*         addr,
                                  bool                bypass,
                                  char**              env) // carries auth info
 {
-  size_t host_len;
-  const char* port = strchr (addr, ':');
-
-  if (port)
-  {
-    port += 1;
-    host_len = port - addr;
-  }
-  else
-  {
-    port = "";
-    host_len = strlen (addr) + 1;
-  }
-
-  char *host= static_cast<char *>(alloca(host_len));
-
-  strncpy (host, addr, host_len - 1);
-  host[host_len - 1] = '\0';
-
   int const cmd_len= 4096;
   wsp::string  cmd_str(cmd_len);
 
@@ -984,14 +976,13 @@ static int sst_donate_mysqldump (const char*         addr,
 
   int ret= snprintf (cmd_str(), cmd_len,
                      "wsrep_sst_mysqldump "
-                     WSREP_SST_OPT_HOST" '%s' "
-                     WSREP_SST_OPT_PORT" '%s' "
+                     WSREP_SST_OPT_ADDR" '%s' "
                      WSREP_SST_OPT_LPORT" '%u' "
                      WSREP_SST_OPT_SOCKET" '%s' "
                      WSREP_SST_OPT_CONF" '%s' "
                      WSREP_SST_OPT_GTID" '%s:%lld'"
                      "%s",
-                     host, port, mysqld_port, mysqld_unix_port,
+                     addr, mysqld_port, mysqld_unix_port,
                      wsrep_defaults_file, uuid_str,
                      (long long)seqno, bypass ? " " WSREP_SST_OPT_BYPASS : "");
 
@@ -1001,7 +992,7 @@ static int sst_donate_mysqldump (const char*         addr,
     return (ret < 0 ? ret : -EMSGSIZE);
   }
 
-  WSREP_DEBUG("Running: '%s'", cmd_str());
+  WSREP_INFO("Initiating SST/IST transfer on DONOR side (%s)", cmd_str());
 
   ret= sst_run_shell (cmd_str(), env, 3);
 
@@ -1009,7 +1000,6 @@ static int sst_donate_mysqldump (const char*         addr,
 
   wsrep->sst_sent (wsrep, &state_id, ret);
 
-
   return ret;
 }
 
@@ -1363,6 +1353,9 @@ wsrep_cb_status_t wsrep_sst_donate_cb (void* app_ctx, void* recv_ctx,
 
   if (!strcmp (WSREP_SST_MYSQLDUMP, method))
   {
+    WSREP_WARN("Percona-XtraDB-Cluster has deprecated SST through mysqldump."
+               " Percona-XtraDB-Cluster recommends using xtrabackup."
+               " Please switch to use xtrabackup or rsync.");
     ret = sst_donate_mysqldump(data, &current_gtid->uuid, uuid_str,
                                current_gtid->seqno, bypass, env());
   }
