@@ -1,4 +1,4 @@
-/* Copyright (c) 2013, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2013, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -318,6 +318,22 @@ bool Sql_cmd_xa_commit::trans_xa_commit(THD *thd)
     */
     res= xs->xa_trans_rolled_back();
 
+#ifdef HAVE_PSI_TRANSACTION_INTERFACE
+    /*
+      If the original transaction is not rolled back then initiate a new PSI
+      transaction to update performance schema related information.
+     */
+    if (!res)
+    {
+      thd->m_transaction_psi= MYSQL_START_TRANSACTION(&thd->m_transaction_state,
+                                                      NULL, NULL, thd->tx_isolation,
+                                                      thd->tx_read_only, false);
+      gtid_set_performance_schema_values(thd);
+      MYSQL_SET_TRANSACTION_XID(thd->m_transaction_psi,
+                                (const void *)xs->get_xid(),
+                                (int)xs->get_state());
+    }
+#endif
     /*
       xs' is_binlogged() is passed through xid_state's member to low-level
       logging routines for deciding how to log.  The same applies to
@@ -327,16 +343,56 @@ bool Sql_cmd_xa_commit::trans_xa_commit(THD *thd)
       xid_state->set_binlogged();
     else
       xid_state->unset_binlogged();
+
+    /*
+      Acquire metadata lock which will ensure that COMMIT is blocked
+      by active FLUSH TABLES WITH READ LOCK (and vice versa COMMIT in
+      progress blocks FTWRL).
+
+      We allow FLUSHer to COMMIT; we assume FLUSHer knows what it does.
+    */
+    MDL_request mdl_request;
+    MDL_REQUEST_INIT(&mdl_request,
+                     MDL_key::COMMIT, "", "", MDL_INTENTION_EXCLUSIVE,
+                     MDL_STATEMENT);
+    if (thd->mdl_context.acquire_lock(&mdl_request,
+                                      thd->variables.lock_wait_timeout))
+    {
+      /*
+        We can't rollback an XA transaction on lock failure due to
+        Innodb redo log and bin log update is involved in rollback.
+        Return error to user for a retry.
+      */
+      my_error(ER_XA_RETRY, MYF(0));
+      DBUG_RETURN(true);
+    }
+
     /* Do not execute gtid wrapper whenever 'res' is true (rm error) */
     gtid_error= MY_TEST(commit_owned_gtids(thd,
                                            true, &need_clear_owned_gtid));
     if (gtid_error)
       my_error(ER_XA_RBROLLBACK, MYF(0));
     res= res || gtid_error;
+
     // todo xa framework: return an error
     ha_commit_or_rollback_by_xid(thd, m_xid, !res);
     xid_state->unset_binlogged();
 
+#ifdef HAVE_PSI_TRANSACTION_INTERFACE
+    if (!res)
+    {
+      if (thd->m_transaction_psi)
+      {
+        /*
+          Set the COMMITTED state in PSI context at the end of committing the
+          XA transaction.
+        */
+        MYSQL_COMMIT_TRANSACTION(thd->m_transaction_psi);
+        thd->m_transaction_psi= NULL;
+      }
+    }
+#endif
+
     transaction_cache_delete(transaction);
     gtid_state_commit_or_rollback(thd, need_clear_owned_gtid, !gtid_error);
     DBUG_RETURN(res);
@@ -374,11 +430,21 @@ bool Sql_cmd_xa_commit::trans_xa_commit(THD *thd)
     */
     MDL_REQUEST_INIT(&mdl_request,
                      MDL_key::COMMIT, "", "", MDL_INTENTION_EXCLUSIVE,
-                     MDL_TRANSACTION);
+                     MDL_STATEMENT);
+    if (thd->mdl_context.acquire_lock(&mdl_request,
+                                      thd->variables.lock_wait_timeout))
+    {
+      /*
+        We can't rollback an XA transaction on lock failure due to
+        Innodb redo log and bin log update are involved in rollback.
+        Return error to user for a retry.
+      */
+      my_error(ER_XA_RETRY, MYF(0));
+      DBUG_RETURN(true);
+    }
 
     gtid_error= MY_TEST(commit_owned_gtids(thd, true, &need_clear_owned_gtid));
-    if (gtid_error || thd->mdl_context.acquire_lock(&mdl_request,
-                                             thd->variables.lock_wait_timeout))
+    if (gtid_error)
     {
 #ifdef WITH_WSREP
       wsrep_register_hton(thd, TRUE);
@@ -510,6 +576,29 @@ bool Sql_cmd_xa_rollback::trans_xa_rollback(THD *thd)
     bool gtid_error= false;
 
     DBUG_ASSERT(xs->is_in_recovery());
+
+    /*
+      Acquire metadata lock which will ensure that XA ROLLBACK is blocked
+      by active FLUSH TABLES WITH READ LOCK (and vice versa ROLLBACK in
+      progress blocks FTWRL). This is to avoid binlog and redo entries
+      while a backup is in progress.
+    */
+    MDL_request mdl_request;
+    MDL_REQUEST_INIT(&mdl_request,
+                     MDL_key::COMMIT, "", "", MDL_INTENTION_EXCLUSIVE,
+                     MDL_STATEMENT);
+    if (thd->mdl_context.acquire_lock(&mdl_request,
+                                      thd->variables.lock_wait_timeout))
+    {
+      /*
+        We can't rollback an XA transaction on lock failure due to
+        Innodb redo log and bin log update is involved in rollback.
+        Return error to user for a retry.
+      */
+      my_error(ER_XAER_RMERR, MYF(0));
+      DBUG_RETURN(true);
+    }
+
     /*
       Like in the commit case a failure to store gtid is regarded
       as the resource manager issue.
@@ -535,6 +624,28 @@ bool Sql_cmd_xa_rollback::trans_xa_rollback(THD *thd)
     DBUG_RETURN(true);
   }
 
+  /*
+    Acquire metadata lock which will ensure that XA ROLLBACK is blocked
+    by active FLUSH TABLES WITH READ LOCK (and vice versa ROLLBACK in
+    progress blocks FTWRL). This is to avoid binlog and redo entries
+    while a backup is in progress.
+  */
+  MDL_request mdl_request;
+  MDL_REQUEST_INIT(&mdl_request,
+                   MDL_key::COMMIT, "", "", MDL_INTENTION_EXCLUSIVE,
+                   MDL_STATEMENT);
+  if (thd->mdl_context.acquire_lock(&mdl_request,
+                                    thd->variables.lock_wait_timeout))
+  {
+    /*
+      We can't rollback an XA transaction on lock failure due to
+      Innodb redo log and bin log update is involved in rollback.
+      Return error to user for a retry.
+    */
+    my_error(ER_XAER_RMERR, MYF(0));
+    DBUG_RETURN(true);
+  }
+
   bool gtid_error= MY_TEST(commit_owned_gtids(thd, true,
                                               &need_clear_owned_gtid));
   bool res= xa_trans_force_rollback(thd) || gtid_error;
@@ -717,30 +828,52 @@ bool Sql_cmd_xa_prepare::trans_xa_prepare(THD *thd)
     my_error(ER_XAER_RMFAIL, MYF(0), xid_state->state_name());
   else if (!xid_state->has_same_xid(m_xid))
     my_error(ER_XAER_NOTA, MYF(0));
-  else if (ha_prepare(thd))
-  {
-#ifdef HAVE_PSI_TRANSACTION_INTERFACE
-    DBUG_ASSERT(thd->m_transaction_psi == NULL);
-#endif
-
-    /*
-      Reset rm_error in case ha_prepare() returned error,
-      so thd->transaction.xid structure gets reset
-      by THD::transaction::cleanup().
-    */
-    thd->get_transaction()->xid_state()->reset_error();
-    cleanup_trans_state(thd);
-    xid_state->set_state(XID_STATE::XA_NOTR);
-    thd->get_transaction()->cleanup();
-    my_error(ER_XA_RBROLLBACK, MYF(0));
-  }
   else
   {
-    xid_state->set_state(XID_STATE::XA_PREPARED);
-    MYSQL_SET_TRANSACTION_XA_STATE(thd->m_transaction_psi,
-                                   (int)xid_state->get_state());
-    if (thd->rpl_thd_ctx.session_gtids_ctx().notify_after_xa_prepare(thd))
-      sql_print_warning("Failed to collect GTID to send in the response packet!");
+    /*
+      Acquire metadata lock which will ensure that XA PREPARE is blocked
+      by active FLUSH TABLES WITH READ LOCK (and vice versa PREPARE in
+      progress blocks FTWRL). This is to avoid binlog and redo entries
+      while a backup is in progress.
+    */
+    MDL_request mdl_request;
+    MDL_REQUEST_INIT(&mdl_request,
+                     MDL_key::COMMIT, "", "", MDL_INTENTION_EXCLUSIVE,
+                     MDL_STATEMENT);
+    if (thd->mdl_context.acquire_lock(&mdl_request,
+                                      thd->variables.lock_wait_timeout) ||
+        ha_prepare(thd))
+    {
+      /*
+        Rollback the transaction if lock failed. For ha_prepare() failure
+        scenarios, transaction is already rolled back by ha_prepare().
+      */
+      if (!mdl_request.ticket)
+        ha_rollback_trans(thd, true);
+
+#ifdef HAVE_PSI_TRANSACTION_INTERFACE
+      DBUG_ASSERT(thd->m_transaction_psi == NULL);
+#endif
+
+      /*
+        Reset rm_error in case ha_prepare() returned error,
+        so thd->transaction.xid structure gets reset
+        by THD::transaction::cleanup().
+      */
+      thd->get_transaction()->xid_state()->reset_error();
+      cleanup_trans_state(thd);
+      xid_state->set_state(XID_STATE::XA_NOTR);
+      thd->get_transaction()->cleanup();
+      my_error(ER_XA_RBROLLBACK, MYF(0));
+    }
+    else
+    {
+      xid_state->set_state(XID_STATE::XA_PREPARED);
+      MYSQL_SET_TRANSACTION_XA_STATE(thd->m_transaction_psi,
+                                     (int)xid_state->get_state());
+      if (thd->rpl_thd_ctx.session_gtids_ctx().notify_after_xa_prepare(thd))
+        sql_print_warning("Failed to collect GTID to send in the response packet!");
+    }
   }
 
   DBUG_RETURN(thd->is_error() ||
@@ -1230,10 +1363,22 @@ bool applier_reset_xa_trans(THD *thd)
   trn_ctx->set_no_2pc(Transaction_ctx::SESSION, false);
   trn_ctx->cleanup();
 #ifdef HAVE_PSI_TRANSACTION_INTERFACE
-  MYSQL_COMMIT_TRANSACTION(thd->m_transaction_psi);
   thd->m_transaction_psi= NULL;
 #endif
   thd->mdl_context.release_transactional_locks();
+  /*
+    On client sessions a XA PREPARE will always be followed by a XA COMMIT
+    or a XA ROLLBACK, and both statements will reset the tx isolation level
+    and access mode when the statement is finishing a transaction.
+
+    For replicated workload it is possible to have other transactions between
+    the XA PREPARE and the XA [COMMIT|ROLLBACK].
+
+    So, if the slave applier changed the current transaction isolation level,
+    it needs to be restored to the session default value after having the
+    XA transaction prepared.
+  */
+  trans_reset_one_shot_chistics(thd);
 
   return thd->is_error();
 }
