@@ -1,20 +1,13 @@
 /* Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
 
   This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License, version 2.0,
-  as published by the Free Software Foundation.
-
-  This program is also distributed with certain software (including
-  but not limited to OpenSSL) that is licensed under separate terms,
-  as designated in a particular file or component or in included license
-  documentation.  The authors of MySQL hereby grant you an additional
-  permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  it under the terms of the GNU General Public License as published by
+  the Free Software Foundation; version 2 of the License.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License, version 2.0, for more details.
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -22,13 +15,13 @@
 
 
 #define MYSQL_SERVER 1
-#include "sql_priv.h"
-#include "unireg.h"
 #include "probes_mysql.h"
 #include "ha_blackhole.h"
 #include "sql_class.h"                          // THD, SYSTEM_THREAD_SLAVE_*
 #include "rpl_rli.h" // THD::rli_slave::rows_query_ev
 
+static PSI_memory_key bh_key_memory_blackhole_share;
+
 static inline bool is_slave_applier(const THD &thd)
 {
   return thd.system_thread == SYSTEM_THREAD_SLAVE_SQL ||
@@ -38,13 +31,10 @@ static inline bool is_slave_applier(const THD &thd)
 static inline bool pretend_for_slave(const THD &thd)
 {
   return is_slave_applier(thd) &&
-#ifdef WITH_WSREP
-    ((thd.rli_slave && thd.rli_slave->rows_query_ev) || thd.query() == NULL);
-#else
-    (thd.rli_slave->rows_query_ev || thd.query() == NULL);
-#endif
+    ((thd.rli_slave && thd.rli_slave->rows_query_ev) || thd.query().str == NULL);
 }
 
+
 /* Static declarations for handlerton */
 
 static handler *blackhole_create_handler(handlerton *hton,
@@ -167,50 +157,7 @@ int ha_blackhole::rnd_next(uchar *buf)
                        TRUE);
   THD *thd= ha_thd();
   if (pretend_for_slave(*thd))
-  {
-    /*
-      Unlike a normal storage engine (e.g. 'InnoDB') in which
-      'rnd_next()' overload actually reads all data in BLOB fields
-      from real storage into internal SE memory (like 'mem_heap' in InnoDB)
-      and updates packed representation ('table->record[0]') of the BLOB
-      field values with pointers to this internal SE memory, 'Blackhole'
-      engine does not do this.
-
-      In case when a database with Blackhole tables serves just as an
-      intermediate binlog server in a replication chain, this may cause
-      data corruption.
-
-      In particular, when 'Update_rows_log_event' is processed on a
-      Blackhole table, calling 'Update_rows_log_event::do_exec_row()' will
-      first copy Before Image (BI) found in 'record[0]' into 'record[1]' and
-      then unpack After Image (AI) into 'record[0]'. The problem is that this
-      record copying is shallow (just 'memcpy()') and for packed BLOB fields
-      it just copies pointer values. In other words, before calling
-      'unpack_current_row()' we end up in a situation when packed BLOB field
-      values in 'record[0]' (AI) and 'record[1]' (BI) point to exactly the
-      same memory location and calling 'unpack_current_row()' for AI
-      overwrites BLOB data in BI.
-
-      To prevent this we do the same trick as for virtual generated columns
-      in 5.7 - keeping old BLOB value inside 'old_value' field in
-      'Field_blob' class by calling 'keep_old_value()' for all BLOB fields
-      currently marked for update in 'table->write_set'..
-    */
-
-    if (table_share->blob_fields != 0)
-      for (Field **field_ptr= table->field; *field_ptr != NULL; ++field_ptr)
-      {
-        Field *current_field= *field_ptr;
-        if ((current_field->flags & BLOB_FLAG) != 0)
-        {
-          Field_blob* bfield= static_cast<Field_blob*>(current_field);
-          if (bfield->in_write_set())
-            bfield->keep_old_value();
-        }
-      }
-
     rc= 0;
-  }
   else
     rc= HA_ERR_END_OF_FILE;
   MYSQL_READ_ROW_DONE(rc);
@@ -405,13 +352,14 @@ static st_blackhole_share *get_share(const char *table_name)
         my_hash_search(&blackhole_open_tables,
                        (uchar*) table_name, length)))
   {
-    if (!(share= (st_blackhole_share*) my_malloc(sizeof(st_blackhole_share) +
+    if (!(share= (st_blackhole_share*) my_malloc(bh_key_memory_blackhole_share,
+                                                 sizeof(st_blackhole_share) +
                                                  length,
                                                  MYF(MY_WME | MY_ZEROFILL))))
       goto error;
 
     share->table_name_length= length;
-    strmov(share->table_name, table_name);
+    my_stpcpy(share->table_name, table_name);
     
     if (my_hash_insert(&blackhole_open_tables, (uchar*) share))
     {
@@ -458,6 +406,11 @@ static PSI_mutex_info all_blackhole_mutexes[]=
   { &bh_key_mutex_blackhole, "blackhole", PSI_FLAG_GLOBAL}
 };
 
+static PSI_memory_info all_blackhole_memory[]=
+{
+  { &bh_key_memory_blackhole_share, "blackhole_share", 0}
+};
+
 void init_blackhole_psi_keys()
 {
   const char* category= "blackhole";
@@ -465,6 +418,9 @@ void init_blackhole_psi_keys()
 
   count= array_elements(all_blackhole_mutexes);
   mysql_mutex_register(category, all_blackhole_mutexes, count);
+
+  count= array_elements(all_blackhole_memory);
+  mysql_memory_register(category, all_blackhole_memory, count);
 }
 #endif
 
@@ -486,7 +442,8 @@ static int blackhole_init(void *p)
                    &blackhole_mutex, MY_MUTEX_INIT_FAST);
   (void) my_hash_init(&blackhole_open_tables, system_charset_info,32,0,0,
                       (my_hash_get_key) blackhole_get_key,
-                      (my_hash_free_key) blackhole_free_key, 0);
+                      (my_hash_free_key) blackhole_free_key, 0,
+                      bh_key_memory_blackhole_share);
 
   return 0;
 }
