@@ -346,8 +346,8 @@ buf_parallel_dblwr_make_path(void)
 	if (parallel_dblwr_buf.path)
 		return(DB_SUCCESS);
 
-	char path[FN_REFLEN];
-	const char *dir;
+	char path[FN_REFLEN + 1 /* OS_PATH_SEPARATOR */];
+	const char *dir = NULL;
 
 	ut_ad(srv_parallel_doublewrite_path);
 
@@ -658,7 +658,7 @@ buf_dblwr_init_or_load_pages(
 
 		os_file_set_nocache(parallel_dblwr_buf.file,
 				    parallel_dblwr_buf.path,
-				    "open");
+				    "open", false);
 
 		os_offset_t size = os_file_get_size(parallel_dblwr_buf.file);
 
@@ -1262,21 +1262,8 @@ buf_dblwr_flush_buffered_writes(
 	srv_stats.dblwr_pages_written.add(dblwr_shard->first_free);
 	srv_stats.dblwr_writes.inc();
 
-	/* Now flush the doublewrite buffer data to disk, unless
-	innodb_flush_method is one of O_SYNC, O_DIRECT_NO_FSYNC, or
-	ALL_O_DIRECT. */
-	switch (srv_unix_file_flush_method) {
-	case SRV_UNIX_NOSYNC:
-	case SRV_UNIX_O_DSYNC:
-	case SRV_UNIX_O_DIRECT_NO_FSYNC:
-	case SRV_UNIX_ALL_O_DIRECT:
-		break;
-	case SRV_UNIX_FSYNC:
-	case SRV_UNIX_LITTLESYNC:
-	case SRV_UNIX_O_DIRECT:
+	if (parallel_dblwr_buf.needs_flush)
 		os_file_flush(parallel_dblwr_buf.file);
-		break;
-	}
 
 	/* We know that the writes have been flushed to disk now
 	and in recovery we will find them in the doublewrite buffer
@@ -1530,8 +1517,23 @@ buf_parallel_dblwr_file_create(void)
 		return(DB_ERROR);
 	}
 
-	os_file_set_nocache(parallel_dblwr_buf.file, parallel_dblwr_buf.path,
-			    "create");
+	const bool o_direct_set
+		= os_file_set_nocache(parallel_dblwr_buf.file,
+				      parallel_dblwr_buf.path,
+				      "create", false);
+	switch (srv_unix_file_flush_method) {
+	case SRV_UNIX_NOSYNC:
+	case SRV_UNIX_O_DSYNC:
+	case SRV_UNIX_O_DIRECT_NO_FSYNC:
+	case SRV_UNIX_ALL_O_DIRECT:
+		parallel_dblwr_buf.needs_flush = !o_direct_set;
+		break;
+	case SRV_UNIX_FSYNC:
+	case SRV_UNIX_LITTLESYNC:
+	case SRV_UNIX_O_DIRECT:
+		parallel_dblwr_buf.needs_flush = true;
+		break;
+	}
 
 	success = os_file_set_size(parallel_dblwr_buf.path,
 				   parallel_dblwr_buf.file, size, false);
