@@ -216,6 +216,7 @@ public:
 
 	ha_rows estimate_rows_upper_bound();
 
+	virtual void adjust_create_info_for_frm(HA_CREATE_INFO *create_info);
 	void update_create_info(HA_CREATE_INFO* create_info);
 
 	int create(
@@ -616,6 +617,7 @@ extern "C" void wsrep_thd_set_query_state(
 	THD *thd, enum wsrep_query_state state);
 extern "C" void wsrep_thd_set_conflict_state(
         THD *thd, bool lock, enum wsrep_conflict_state state);
+extern "C" void wsrep_thd_mark_split_trx(THD* thd, bool mini_trx);
 
 extern "C" void wsrep_thd_set_trx_to_replay(THD *thd, uint64 trx_id);
 
@@ -794,6 +796,9 @@ public:
 	/** Validate COMPRESSION option. */
 	bool create_option_compression_is_valid();
 
+	/** Validate ENCRYPTION option. */
+	bool create_option_encryption_is_valid() const;
+
 	/** Prepare to create a table. */
 	int prepare_create_table(const char* name);
 
@@ -1168,7 +1173,7 @@ innobase_build_index_translation(
 
 /** Compression dictionary id container */
 typedef std::map<uint16, ulint, std::less<uint16>,
-	ut_allocator<std::pair<uint16, const ulint> > >
+	ut_allocator<std::pair<const uint16, ulint> > >
 	zip_dict_id_container_t;
 
 /** This function checks if all the compression dictionaries referenced
