@@ -448,12 +448,18 @@ struct Encryption {
         static const char* to_string(Type type)
 		MY_ATTRIBUTE((warn_unused_result));
 
-        /** Check if the string is "empty" or "none".
-        @param[in]      algorithm       Encryption algorithm to check
-        @return true if no algorithm requested */
+	/** Check if the string is "" or "n".
+	@param[in]      algorithm       Encryption algorithm to check
+	@return true if no algorithm requested */
 	static bool is_none(const char* algorithm)
 		MY_ATTRIBUTE((warn_unused_result));
 
+	/** Check if the NO algorithm was explicitly specified.
+	@param[in]      algorithm       Encryption algorithm to check
+	@return true if no algorithm explicitly requested */
+	static bool none_explicitly_specified(const char* algorithm)
+		MY_ATTRIBUTE((warn_unused_result));
+
         /** Generate random encryption value for key and iv.
         @param[in,out]	value	Encryption value */
 	static void random_value(byte* value);
@@ -514,6 +520,11 @@ struct Encryption {
 		ulint			dst_len)
 		MY_ATTRIBUTE((warn_unused_result));
 
+#ifndef UNIV_INNOCHECKSUM
+	/** Check if keyring plugin loaded. */
+	static bool MY_ATTRIBUTE((warn_unused_result)) check_keyring();
+#endif
+
 	/** Encrypt type */
 	Type			m_type;
 
@@ -1197,12 +1208,12 @@ do {									\
 		state, locker, key, op, name,				\
 		src_file, static_cast<uint>(src_line))			\
 
-# define register_pfs_file_rename_end(locker, result)			\
+# define register_pfs_file_rename_end(locker, from, to, result)		\
 do {									\
 	if (locker != NULL) {						\
 		 PSI_FILE_CALL(						\
-			end_file_open_wait)(				\
-			locker, result);				\
+			end_file_rename_wait)(				\
+			locker, from, to, result);			\
 	}								\
 }while(0)
 
@@ -1286,10 +1297,11 @@ The wrapper functions have the prefix of "innodb_". */
 	pfs_os_file_close_no_error_handling_func(file, __FILE__, __LINE__)
 
 # define os_aio(type, mode, name, file, buf, offset,			\
-		n, read_only, message1, message2, space_id, trx)	\
+		n, read_only, message1, message2, space_id, trx,	\
+		should_buffer)	\
 	pfs_os_aio_func(type, mode, name, file, buf, offset,		\
 			n, read_only, message1, message2, space_id,	\
-			trx, __FILE__, __LINE__)
+			trx, should_buffer, __FILE__, __LINE__)
 
 # define os_file_read_pfs(type, file, buf, offset, n)			\
 	pfs_os_file_read_func(type, file, buf, offset, n, NULL,		\
@@ -1549,6 +1561,12 @@ an asynchronous I/O operation.
 @param[in,out]	m2		message for the AIO handler (can be used to
 				identify a completed AIO operation); ignored
 				if mode is OS_AIO_SYNC
+@param[in]	should_buffer	Whether to buffer an aio request.
+				AIO read ahead uses this. If you plan to
+				use this parameter, make sure you remember to
+				call os_aio_dispatch_read_array_submit()
+				when you're ready to commit all your
+				requests.
 @param[in]	src_file	file name where func invoked
 @param[in]	src_line	line where the func invoked
 @return DB_SUCCESS if request was queued successfully, FALSE if fail */
@@ -1567,6 +1585,7 @@ pfs_os_aio_func(
 	void*		m2,
 	ulint		space_id,
 	trx_t*		trx,
+	bool		should_buffer,
 	const char*	src_file,
 	ulint		src_line);
 
@@ -1737,9 +1756,11 @@ to original un-instrumented file I/O APIs */
 	os_file_close_no_error_handling_func(file)
 
 # define os_aio(type, mode, name, file, buf, offset,			\
-		n, read_only, message1, message2, space_id, trx)	\
+		n, read_only, message1, message2, space_id, trx,	\
+		should_buffer)	\
 	os_aio_func(type, mode, name, file, buf, offset,		\
-		n, read_only, message1, message2, space_id, trx)
+		n, read_only, message1, message2, space_id, trx,	\
+		should_buffer)
 
 # define os_file_read_pfs(type, file, buf, offset, n)			\
 	os_file_read_func(type, file, buf, offset, n)
@@ -2089,6 +2110,12 @@ Requests an asynchronous i/o operation.
 @param[in,out]	m2		message for the AIO handler (can be used to
 				identify a completed AIO operation); ignored
 				if mode is OS_AIO_SYNC
+@param[in]	should_buffer	Whether to buffer an aio request.
+				AIO read ahead uses this. If you plan to
+				use this parameter, make sure you remember to
+				call os_aio_dispatch_read_array_submit()
+				when you're ready to commit all your
+				requests.
 @return DB_SUCCESS or error code */
 dberr_t
 os_aio_func(
@@ -2103,7 +2130,8 @@ os_aio_func(
 	fil_node_t*	m1,
 	void*		m2,
 	ulint		space_id,
-	trx_t*		trx);
+	trx_t*		trx,
+	bool		should_buffer);
 
 /** Wakes up all async i/o threads so that they know to exit themselves in
 shutdown. */
@@ -2295,6 +2323,10 @@ is_absolute_path(
 	return(false);
 }
 
+/** Submit buffered AIO requests on the given segment to the kernel. */
+void
+os_aio_dispatch_read_array_submit();
+
 #ifndef UNIV_NONINL
 #include "os0file.ic"
 #endif /* UNIV_NONINL */
