@@ -1235,6 +1235,19 @@ srv_open_tmp_tablespace(
 		it stays open until shutdown. */
 		if (fil_space_open(tmp_space->name())) {
 
+			if (srv_tmp_tablespace_encrypt) {
+
+				fil_space_t*	space =
+					fil_space_get(temp_space_id);
+				space->flags |= FSP_FLAGS_MASK_ENCRYPTION;
+
+				err = fil_set_encryption(space->id,
+							 Encryption::AES,
+							 NULL,
+							 NULL);
+				ut_a(err == DB_SUCCESS);
+			}
+
 			/* Initialize the header page */
 			mtr_start(&mtr);
 			mtr_set_log_mode(&mtr, MTR_LOG_NO_REDO);
@@ -1871,7 +1884,8 @@ innobase_start_or_create_for_mysql(void)
 		<< size << unit << ", instances = " << srv_buf_pool_instances
 		<< ", chunk size = " << chunk_size << chunk_unit;
 
-	err = buf_pool_init(srv_buf_pool_size, srv_buf_pool_instances);
+	err = buf_pool_init(srv_buf_pool_size, static_cast<bool>(srv_numa_interleave),
+	                    srv_buf_pool_instances);
 
 	if (err != DB_SUCCESS) {
 		ib::error() << "Cannot allocate memory for the buffer pool";
@@ -2591,21 +2605,31 @@ files_checked:
 	running in single threaded mode essentially. Only the IO threads
 	should be running at this stage. */
 
+	/* Deprecate innodb_undo_logs.  But still use it if it is set to
+	non-default and innodb_rollback_segments is default. */
+	ut_a(srv_rollback_segments > 0);
+	ut_a(srv_rollback_segments <= TRX_SYS_N_RSEGS);
 	ut_a(srv_undo_logs > 0);
 	ut_a(srv_undo_logs <= TRX_SYS_N_RSEGS);
+	if (srv_undo_logs < TRX_SYS_N_RSEGS) {
+		ib::warn() << deprecated_undo_logs;
+		if (srv_rollback_segments == TRX_SYS_N_RSEGS) {
+			srv_rollback_segments = srv_undo_logs;
+		}
+	}
 
 	/* The number of rsegs that exist in InnoDB is given by status
 	variable srv_available_undo_logs. The number of rsegs to use can
-	be set using the dynamic global variable srv_undo_logs. */
+	be set using the dynamic global variable srv_rollback_segments. */
 
 	srv_available_undo_logs = trx_sys_create_rsegs(
-		srv_undo_tablespaces, srv_undo_logs, srv_tmp_undo_logs);
+		srv_undo_tablespaces, srv_rollback_segments, srv_tmp_undo_logs);
 
 	if (srv_available_undo_logs == ULINT_UNDEFINED) {
 		/* Can only happen if server is read only. */
 		ut_a(srv_read_only_mode);
-		srv_undo_logs = ULONG_UNDEFINED;
-	} else if (srv_available_undo_logs < srv_undo_logs
+		srv_rollback_segments = ULONG_UNDEFINED;
+	} else if (srv_available_undo_logs < srv_rollback_segments
 		   && !srv_force_recovery && !recv_needed_recovery) {
 		ib::error() << "System or UNDO tablespace is running of out"
 			    << " of space";
@@ -2682,6 +2706,9 @@ files_checked:
 		srv_start_state_set(SRV_START_STATE_MASTER);
 	}
 
+	/* Enable row log encryption if it is set */
+	log_tmp_enable_encryption_if_set();
+
 	if (!srv_read_only_mode
 	    && srv_force_recovery < SRV_FORCE_NO_BACKGROUND) {
 
