@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2015, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2017, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -316,7 +316,6 @@ trx_sys_print_mysql_binlog_offset(void)
 
 #ifdef WITH_WSREP
 
-#ifdef UNIV_DEBUG
 static long long trx_sys_cur_xid_seqno = -1;
 static unsigned char trx_sys_cur_xid_uuid[16];
 
@@ -334,16 +333,18 @@ void read_wsrep_xid_uuid(const XID* xid, unsigned char* buf)
     memcpy(buf, xid->get_data() + 8, 16);
 }
 
-#endif /* UNIV_DEBUG */
 
 void
 trx_sys_update_wsrep_checkpoint(
         const XID*      xid,        /*!< in: transaction XID */
         trx_sysf_t*     sys_header, /*!< in: sys_header */
-        mtr_t*          mtr)        /*!< in: mtr */
+        mtr_t*          mtr,        /*!< in: mtr */
+        bool            recovery)   /*!< in: running recovery */
 {
 
-#ifdef UNIV_DEBUG
+#ifndef UNIV_DEBUG
+        if (recovery)
+#endif /* !UNIV_DEBUG */
         {
             /* Check that seqno is monotonically increasing */
             unsigned char xid_uuid[16];
@@ -351,16 +352,37 @@ trx_sys_update_wsrep_checkpoint(
             read_wsrep_xid_uuid(xid, xid_uuid);
             if (!memcmp(xid_uuid, trx_sys_cur_xid_uuid, 8))
             {
-                ut_ad(xid_seqno > trx_sys_cur_xid_seqno);
-                trx_sys_cur_xid_seqno = xid_seqno;
+                if (recovery)
+                {
+                    /* When recovery happens prepare transactions
+                    are revived based on undo-log entries in InnoDB.
+                    This order may not match with the commit order
+                    logged to binlog.
+                    Sequence matching is not needed for MySQL
+                    as standalone. Aim is to just ensure that all
+                    prepare stage transaction are marked as committed.
+                    But in PXC the commit order of recover transaction
+                    should be same as it would be if transaction are running
+                    normally or we need to ensure that only the latest seen
+                    xid is updated and persisted as wsrep recover position
+                    co-ordinates. */
+                    if (xid_seqno > trx_sys_cur_xid_seqno)
+                        trx_sys_cur_xid_seqno = xid_seqno;
+                    else
+                        return;
+                }
+                else
+                {
+                    ut_ad(xid_seqno > trx_sys_cur_xid_seqno);
+                    trx_sys_cur_xid_seqno = xid_seqno;
+                }
             }
             else
             {
                 memcpy(trx_sys_cur_xid_uuid, xid_uuid, 16);
+                trx_sys_cur_xid_seqno = xid_seqno;
             }
-            trx_sys_cur_xid_seqno = xid_seqno;
         }
-#endif /* UNIV_DEBUG */
 
         ut_ad(xid && mtr && sys_header);
         ut_a(xid->get_format_id() == -1 || wsrep_is_wsrep_xid(xid));
@@ -1049,20 +1071,29 @@ trx_sys_create_rsegs(
 		ulint	new_rsegs = n_rsegs - n_used;
 
 		for (i = 0; i < new_rsegs; ++i) {
-			ulint	space;
+			ulint	space_id;
+			space_id = (n_spaces == 0) ? 0
+				: (srv_undo_space_id_start + i % n_spaces);
 
-			/* Tablespace 0 is the system tablespace. All UNDO
-			log tablespaces start from 1. */
+			ut_ad(n_spaces == 0
+			      || srv_is_undo_tablespace(space_id));
 
-			if (n_spaces > 0) {
-				space = (i % n_spaces) + 1;
-			} else {
-				space = 0; /* System tablespace */
-			}
-
-			if (trx_rseg_create(space, 0) != NULL) {
+			if (trx_rseg_create(space_id, 0) != NULL) {
 				++n_used;
 				++n_redo_active;
+
+				ulint	last_undo_space =
+					srv_undo_space_id_start
+					+ (srv_undo_tablespaces_active - 1);
+
+				/* Increase the number of active undo
+				tablespace in case new rollback segment
+				assigned to new undo tablespace. */
+				if (space_id > last_undo_space) {
+					srv_undo_tablespaces_active++;
+
+					ut_ad(space_id == last_undo_space + 1);
+				}
 			} else {
 				break;
 			}
