@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1996, 2017, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1996, 2018, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -139,6 +139,8 @@ trx_init(
 
 	trx->no = TRX_ID_MAX;
 
+	trx->skip_lock_inheritance = false;
+
 	trx->is_recovered = false;
 
 	trx->op_info = "";
@@ -509,6 +511,7 @@ trx_create_low()
 
 #ifdef WITH_WSREP
 	trx->wsrep_event = NULL;
+	trx->wsrep_recover_xid = NULL;
 #endif /* WITH_WSREP */
 
 	return(trx);
@@ -1762,6 +1765,8 @@ trx_write_serialisation_history(
 	MONITOR_INC(MONITOR_TRX_COMMIT_UNDO);
 
 #ifdef WITH_WSREP
+	DBUG_EXECUTE_IF("crash_before_trx_commit_in_memory",
+			{ sleep(3); DBUG_SUICIDE(); });
         sys_header = trx_sysf_get(mtr);
 	/* Update latest MySQL wsrep XID in trx sys header.
 	If given transaction is marked for replay then avoid updating
@@ -1771,6 +1776,12 @@ trx_write_serialisation_history(
         {
             trx_sys_update_wsrep_checkpoint(trx->xid, sys_header, mtr);
         }
+	else if (trx->wsrep_recover_xid)
+	{
+		trx_sys_update_wsrep_checkpoint(
+				trx->wsrep_recover_xid, sys_header, mtr, true);
+		trx->wsrep_recover_xid = NULL;
+	}
 #endif /* WITH_WSREP */
 
 	/* Update the latest MySQL binlog name and offset info
@@ -3180,6 +3191,22 @@ trx_prepare(
 	trx_sys_mutex_exit();
 	/*--------------------------------------*/
 
+	/* Force isolation level to RC and release GAP locks
+	for test purpose. */
+	DBUG_EXECUTE_IF("ib_force_release_gap_lock_prepare",
+			trx->isolation_level = TRX_ISO_READ_COMMITTED;);
+
+	/* Release read locks after PREPARE for READ COMMITTED
+	and lower isolation. */
+	if (trx->isolation_level <= TRX_ISO_READ_COMMITTED) {
+
+		/* Stop inheriting GAP locks. */
+		trx->skip_lock_inheritance = true;
+
+		/* Release only GAP locks for now. */
+		lock_trx_release_read_locks(trx, true);
+	}
+
 	switch (thd_requested_durability(trx->mysql_thd)) {
 	case HA_IGNORE_DURABILITY:
 		/* We set the HA_IGNORE_DURABILITY during prepare phase of
