@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1994, 2017, Oracle and/or its affiliates. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -50,6 +50,10 @@ Created 5/11/1994 Heikki Tuuri
 #include "log.h"
 
 #ifdef _WIN32
+#include <mysql/innodb_priv.h> /* For sql_print_error */
+typedef VOID(WINAPI *time_fn)(LPFILETIME);
+static time_fn ut_get_system_time_as_file_time = GetSystemTimeAsFileTime;
+
 /*****************************************************************//**
 NOTE: The Windows epoch starts from 1601/01/01 whereas the Unix
 epoch starts from 1970/1/1. For selection of constant see:
@@ -57,6 +61,28 @@ http://support.microsoft.com/kb/167296/ */
 #define WIN_TO_UNIX_DELTA_USEC	11644473600000000LL
 
 
+/**
+Initialise highest available time resolution API on Windows
+@return 0 if all OK else -1 */
+int
+ut_win_init_time()
+{
+	HMODULE h = LoadLibrary("kernel32.dll");
+	if (h != NULL)
+	{
+		time_fn pfn = (time_fn)GetProcAddress(h, "GetSystemTimePreciseAsFileTime");
+		if (pfn != NULL)
+		{
+			ut_get_system_time_as_file_time = pfn;
+		}
+		return false;
+	}
+	DWORD error = GetLastError();
+  sql_print_error(
+		"LoadLibrary(\"kernel32.dll\") failed: GetLastError returns %lu", error);
+	return(-1);
+}
+
 /*****************************************************************//**
 This is the Windows version of gettimeofday(2).
 @return 0 if all OK else -1 */
@@ -75,7 +101,7 @@ ut_gettimeofday(
 		return(-1);
 	}
 
-	GetSystemTimeAsFileTime(&ft);
+	ut_get_system_time_as_file_time(&ft);
 
 	tm = (int64_t) ft.dwHighDateTime << 32;
 	tm |= ft.dwLowDateTime;
@@ -932,6 +958,15 @@ fatal_or_error::~fatal_or_error()
 	ut_a(!m_fatal);
 }
 
+warn_or_info::~warn_or_info()
+{
+	if (m_warn) {
+		sql_print_warning("InnoDB: %s", m_oss.str().c_str());
+	} else {
+		sql_print_information("InnoDB: %s", m_oss.str().c_str());
+	}
+}
+
 } // namespace ib
 
 #endif /* !UNIV_INNOCHECKSUM */
