@@ -2,7 +2,7 @@
 #define HA_PARTITION_INCLUDED
 
 /*
-   Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
+   Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -436,7 +436,13 @@ public:
   }
   int rnd_pos_by_record(uchar *record)
   {
-    return Partition_helper::ph_rnd_pos_by_record(record);
+    if (unlikely(get_part_for_delete(record,
+                                     m_table->record[0],
+                                     m_part_info,
+                                     &m_last_part))) {
+      return(HA_ERR_INTERNAL_ERROR);
+    }
+    return(m_file[m_last_part]->rnd_pos_by_record(record));
   }
   void position(const uchar *record)
   {
@@ -1157,6 +1163,9 @@ public:
     void rpl_after_update_rows();
     bool rpl_lookup_rows();
 
+  /* For MyRocks Writebatch Replication validation */
+    bool rpl_can_handle_stm_event() const;
+
   /*
     -------------------------------------------------------------------------
     MODULE partitioning specific handler API
@@ -1197,13 +1206,6 @@ private:
   int rnd_end_in_part(uint part_id, bool scan);
   void position_in_last_part(uchar *ref, const uchar *record);
   int rnd_pos_in_part(uint part_id, uchar *buf, uchar *pos);
-  int rnd_pos_by_record_in_last_part(uchar *row)
-  {
-    /*
-      Not much overhead to use default function. This avoids out-of-sync code.
-    */
-    return handler::rnd_pos_by_record(row);
-  }
   int index_init_in_part(uint part, uint keynr, bool sorted);
   int index_end_in_part(uint part);
   int index_last_in_part(uint part, uchar *buf);
