@@ -1,4 +1,4 @@
-/* Copyright (c) 2000, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2000, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -121,12 +121,18 @@ static DH *get_dh2048(void)
   DH *dh;
   if ((dh=DH_new()))
   {
-    dh->p=BN_bin2bn(dh2048_p,sizeof(dh2048_p),NULL);
-    dh->g=BN_bin2bn(dh2048_g,sizeof(dh2048_g),NULL);
+    BIGNUM* p= BN_bin2bn(dh2048_p,sizeof(dh2048_p),NULL);
+    BIGNUM* g= BN_bin2bn(dh2048_g,sizeof(dh2048_g),NULL);
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+    dh->p= p;
+    dh->g= g;
     if (! dh->p || ! dh->g)
+#else
+    if (!DH_set0_pqg(dh, p, NULL, g))
+#endif
     {
       DH_free(dh);
-      dh=0;
+      dh= NULL;
     }
   }
   return(dh);
@@ -230,6 +236,7 @@ vio_set_cert_stuff(SSL_CTX *ctx, const char *cert_file, const char *key_file,
 
 #ifndef HAVE_YASSL
 /* OpenSSL specific */
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
 
 #ifdef HAVE_PSI_INTERFACE
 static PSI_rwlock_key key_rwlock_openssl;
@@ -389,23 +396,30 @@ static void deinit_lock_callback_functions()
 {
   set_lock_callback_functions(FALSE);
 }
+#endif /* OPENSSL_VERSION_NUMBER < 0x10100000L */
 
 void vio_ssl_end()
 {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
   int i= 0;
+#endif
 
   if (ssl_initialized) {
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     ERR_remove_state(0);
+#endif
     ERR_free_strings();
     EVP_cleanup();
 
     CRYPTO_cleanup_all_ex_data();
 
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
     deinit_lock_callback_functions();
 
     for (; i < CRYPTO_num_locks(); ++i)
       mysql_rwlock_destroy(&openssl_stdlocks[i].lock);
     OPENSSL_free(openssl_stdlocks);
+#endif
 
     ssl_initialized= FALSE;
   }
@@ -423,7 +437,7 @@ void ssl_start()
     OpenSSL_add_all_algorithms();
     SSL_load_error_strings();
 
-#ifndef HAVE_YASSL
+#if !defined(HAVE_YASSL) && (OPENSSL_VERSION_NUMBER < 0x10100000L)
     init_ssl_locks();
     init_lock_callback_functions();
 #endif
@@ -492,6 +506,9 @@ new_VioSSLFd(const char *key_file, const char *cert_file,
   long ssl_ctx_options= SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3;
   int ret_set_cipherlist= 0;
   char cipher_list[SSL_CIPHER_LIST_SIZE]= {0};
+#if !defined(HAVE_YASSL) && (OPENSSL_VERSION_NUMBER < 0x10002000L)
+  EC_KEY *ecdh;
+#endif
   DBUG_ENTER("new_VioSSLFd");
   DBUG_PRINT("enter",
              ("key_file: '%s'  cert_file: '%s'  ca_file: '%s'  ca_path: '%s'  "
@@ -543,12 +560,21 @@ new_VioSSLFd(const char *key_file, const char *cert_file,
   SSL_CTX_set_options(ssl_fd->ssl_context, ssl_ctx_options);
 
   /*
-    Set the ciphers that can be used
+    We explicitly prohibit weak ciphers.
     NOTE: SSL_CTX_set_cipher_list will return 0 if
     none of the provided ciphers could be selected
   */
   DBUG_ASSERT(strlen(tls_cipher_blocked) + 1 <= sizeof(cipher_list));
   strcat(cipher_list, tls_cipher_blocked);
+
+  /*
+    If ciphers are specified explicitly by caller, use them.
+    Otherwise, fallback to the default list.
+
+    In either case, we make sure we stay within the valid bounds.
+    Note that we have already consumed tls_cipher_blocked
+    worth of space.
+  */
   if (cipher)
   {
     if (strlen(cipher_list) + strlen(cipher) + 1 > sizeof(cipher_list))
@@ -665,6 +691,45 @@ new_VioSSLFd(const char *key_file, const char *cert_file,
   }
   DH_free(dh);
 
+#ifndef HAVE_YASSL
+#if OPENSSL_VERSION_NUMBER < 0x10002000L
+  ecdh= EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
+  if (!ecdh)
+  {
+    *error= SSL_INITERR_DHFAIL;
+    DBUG_PRINT("error", ("%s", sslGetErrString(*error)));
+    report_errors();
+    SSL_CTX_free(ssl_fd->ssl_context);
+    my_free(ssl_fd);
+    DBUG_RETURN(0);
+  }
+
+  if (SSL_CTX_set_tmp_ecdh(ssl_fd->ssl_context, ecdh) != 1)
+  {
+    *error= SSL_INITERR_DHFAIL;
+    DBUG_PRINT("error", ("%s", sslGetErrString(*error)));
+    report_errors();
+    EC_KEY_free(ecdh);
+    SSL_CTX_free(ssl_fd->ssl_context);
+    my_free(ssl_fd);
+    DBUG_RETURN(0);
+  }
+  EC_KEY_free(ecdh);
+
+#else /* OPENSSL_VERSION_NUMBER < 0x10002000L */
+
+  if (SSL_CTX_set_ecdh_auto(ssl_fd->ssl_context, 1) != 1)
+  {
+    *error= SSL_INITERR_DHFAIL;
+    DBUG_PRINT("error", ("%s", sslGetErrString(*error)));
+    report_errors();
+    SSL_CTX_free(ssl_fd->ssl_context);
+    my_free(ssl_fd);
+    DBUG_RETURN(0);
+  }
+#endif /* OPENSSL_VERSION_NUMBER < 0x10002000L */
+#endif /* !HAVE_YASSL */
+
   DBUG_PRINT("exit", ("OK 1"));
 
   DBUG_RETURN(ssl_fd);
